# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/materializationjobservice/materialization_job_service.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from google.protobuf import descriptor_pb2 as google_dot_protobuf_dot_descriptor__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from tecton_proto.auth import service_pb2 as tecton__proto_dot_auth_dot_service__pb2
from tecton_proto.materialization import params_pb2 as tecton__proto_dot_materialization_dot_params__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nHtecton_proto/materializationjobservice/materialization_job_service.proto\x12&tecton_proto.materializationjobservice\x1a\x1cgoogle/api/annotations.proto\x1a google/protobuf/descriptor.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1ftecton_proto/auth/service.proto\x1a)tecton_proto/materialization/params.proto\"\xdb\x02\n\x19MaterializationJobRequest\x12\x1c\n\tworkspace\x18\x01 \x01(\tR\tworkspace\x12!\n\x0c\x66\x65\x61ture_view\x18\x02 \x01(\tR\x0b\x66\x65\x61tureView\x12\x39\n\nstart_time\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tstartTime\x12\x35\n\x08\x65nd_time\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x07\x65ndTime\x12\x16\n\x06online\x18\x05 \x01(\x08R\x06online\x12\x18\n\x07offline\x18\x06 \x01(\x08R\x07offline\x12;\n\x1ause_tecton_managed_retries\x18\x07 \x01(\x08R\x17useTectonManagedRetries\x12\x1c\n\toverwrite\x18\x08 \x01(\x08R\toverwrite\"j\n\x1aMaterializationJobResponse\x12L\n\x03job\x18\x01 \x01(\x0b\x32:.tecton_proto.materializationjobservice.MaterializationJobR\x03job\"\xc4\x04\n\x12MaterializationJob\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12\x1c\n\tworkspace\x18\x02 \x01(\tR\tworkspace\x12!\n\x0c\x66\x65\x61ture_view\x18\x03 \x01(\tR\x0b\x66\x65\x61tureView\x12\x39\n\nstart_time\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tstartTime\x12\x35\n\x08\x65nd_time\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x07\x65ndTime\x12\x39\n\ncreated_at\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tcreatedAt\x12\x39\n\nupdated_at\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tupdatedAt\x12\x14\n\x05state\x18\t \x01(\tR\x05state\x12N\n\x08\x61ttempts\x18\n \x03(\x0b\x32\x32.tecton_proto.materializationjobservice.JobAttemptR\x08\x61ttempts\x12\x42\n\x0fnext_attempt_at\x18\x0b \x01(\x0b\x32\x1a.google.protobuf.TimestampR\rnextAttemptAt\x12\x16\n\x06online\x18\x0c \x01(\x08R\x06online\x12\x18\n\x07offline\x18\r \x01(\x08R\x07offline\x12\x19\n\x08job_type\x18\x0e \x01(\tR\x07jobType\"\xc1\x01\n\nJobAttempt\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12\x39\n\ncreated_at\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tcreatedAt\x12\x39\n\nupdated_at\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tupdatedAt\x12\x14\n\x05state\x18\x04 \x01(\tR\x05state\x12\x17\n\x07run_url\x18\x05 \x01(\tR\x06runUrl\"R\n\x0fListJobsRequest\x12\x1c\n\tworkspace\x18\x01 \x01(\tR\tworkspace\x12!\n\x0c\x66\x65\x61ture_view\x18\x02 \x01(\tR\x0b\x66\x65\x61tureView\"b\n\x10ListJobsResponse\x12N\n\x04jobs\x18\x01 \x03(\x0b\x32:.tecton_proto.materializationjobservice.MaterializationJobR\x04jobs\"g\n\rGetJobRequest\x12\x15\n\x06job_id\x18\x01 \x01(\tR\x05jobId\x12\x1c\n\tworkspace\x18\x02 \x01(\tR\tworkspace\x12!\n\x0c\x66\x65\x61ture_view\x18\x03 \x01(\tR\x0b\x66\x65\x61tureView\"^\n\x0eGetJobResponse\x12L\n\x03job\x18\x01 \x01(\x0b\x32:.tecton_proto.materializationjobservice.MaterializationJobR\x03job\"j\n\x10\x43\x61ncelJobRequest\x12\x15\n\x06job_id\x18\x01 \x01(\tR\x05jobId\x12\x1c\n\tworkspace\x18\x02 \x01(\tR\tworkspace\x12!\n\x0c\x66\x65\x61ture_view\x18\x03 \x01(\tR\x0b\x66\x65\x61tureView\"a\n\x11\x43\x61ncelJobResponse\x12L\n\x03job\x18\x01 \x01(\x0b\x32:.tecton_proto.materializationjobservice.MaterializationJobR\x03job\"\x85\x01\n\x19GetLatestReadyTimeRequest\x12\x1c\n\tworkspace\x18\x01 \x01(\tR\tworkspace\x12!\n\x0c\x66\x65\x61ture_view\x18\x02 \x01(\tR\x0b\x66\x65\x61tureView\x12\'\n\x0f\x66\x65\x61ture_service\x18\x03 \x01(\tR\x0e\x66\x65\x61tureService\"\xc8\x01\n\x1aGetLatestReadyTimeResponse\x12S\n\x18online_latest_ready_time\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x15onlineLatestReadyTime\x12U\n\x19offline_latest_ready_time\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x16offlineLatestReadyTime\"w\n+TestOnlyGetMaterializationTaskParamsRequest\x12\x1c\n\tworkspace\x18\x01 \x01(\tR\tworkspace\x12*\n\x11\x66\x65\x61ture_view_name\x18\x02 \x01(\tR\x0f\x66\x65\x61tureViewName\"t\n,TestOnlyGetMaterializationTaskParamsResponse\x12\x44\n\x1e\x65ncoded_materialization_params\x18\x01 \x01(\tR\x1c\x65ncodedMaterializationParams2\xbd\x0b\n\x19MaterializationJobService\x12\x95\x02\n\x18SubmitMaterializationJob\x12\x41.tecton_proto.materializationjobservice.MaterializationJobRequest\x1a\x42.tecton_proto.materializationjobservice.MaterializationJobResponse\"r\x82\xd3\xe4\x93\x02(\"#/v1/jobs/submit-materialization-job:\x01*\xa2\xbc\xe6\xc0\x05>\x1a\r\x08\x01\x12\tworkspace*-tecton/workspace#update_materialization_state\x12\xfd\x01\n\x17ListMaterializationJobs\x12\x37.tecton_proto.materializationjobservice.ListJobsRequest\x1a\x38.tecton_proto.materializationjobservice.ListJobsResponse\"o\x82\xd3\xe4\x93\x02\'\"\"/v1/jobs/list-materialization-jobs:\x01*\xa2\xbc\xe6\xc0\x05<\x1a\r\x08\x01\x12\tworkspace*+tecton/workspace#read_materialization_state\x12\xbb\x01\n\x15GetMaterializationJob\x12\x35.tecton_proto.materializationjobservice.GetJobRequest\x1a\x36.tecton_proto.materializationjobservice.GetJobResponse\"3\x82\xd3\xe4\x93\x02%\" /v1/jobs/get-materialization-job:\x01*\xa2\xbc\xe6\xc0\x05\x02@\x01\x12\xc7\x01\n\x18\x43\x61ncelMaterializationJob\x12\x38.tecton_proto.materializationjobservice.CancelJobRequest\x1a\x39.tecton_proto.materializationjobservice.CancelJobResponse\"6\x82\xd3\xe4\x93\x02(\"#/v1/jobs/cancel-materialization-job:\x01*\xa2\xbc\xe6\xc0\x05\x02@\x01\x12\x88\x02\n\x12GetLatestReadyTime\x12\x41.tecton_proto.materializationjobservice.GetLatestReadyTimeRequest\x1a\x42.tecton_proto.materializationjobservice.GetLatestReadyTimeResponse\"k\x82\xd3\xe4\x93\x02#\"\x1e/v1/jobs/get-latest-ready-time:\x01*\xa2\xbc\xe6\xc0\x05<\x1a\r\x08\x01\x12\tworkspace*+tecton/workspace#read_materialization_state\x12\xf4\x01\n$TestOnlyGetMaterializationTaskParams\x12S.tecton_proto.materializationjobservice.TestOnlyGetMaterializationTaskParamsRequest\x1aT.tecton_proto.materializationjobservice.TestOnlyGetMaterializationTaskParamsResponse\"!\xa2\xbc\xe6\xc0\x05\x1b*\x19tecton/organization#adminB\x85\x01\n$com.tecton.materializationjobserviceB\x1eMaterializationJobServiceProtoP\x01Z;github.com/tecton-ai/tecton_proto/materializationjobservice')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.materializationjobservice.materialization_job_service_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n$com.tecton.materializationjobserviceB\036MaterializationJobServiceProtoP\001Z;github.com/tecton-ai/tecton_proto/materializationjobservice'
  _MATERIALIZATIONJOBSERVICE.methods_by_name['SubmitMaterializationJob']._options = None
  _MATERIALIZATIONJOBSERVICE.methods_by_name['SubmitMaterializationJob']._serialized_options = b'\202\323\344\223\002(\"#/v1/jobs/submit-materialization-job:\001*\242\274\346\300\005>\032\r\010\001\022\tworkspace*-tecton/workspace#update_materialization_state'
  _MATERIALIZATIONJOBSERVICE.methods_by_name['ListMaterializationJobs']._options = None
  _MATERIALIZATIONJOBSERVICE.methods_by_name['ListMaterializationJobs']._serialized_options = b'\202\323\344\223\002\'\"\"/v1/jobs/list-materialization-jobs:\001*\242\274\346\300\005<\032\r\010\001\022\tworkspace*+tecton/workspace#read_materialization_state'
  _MATERIALIZATIONJOBSERVICE.methods_by_name['GetMaterializationJob']._options = None
  _MATERIALIZATIONJOBSERVICE.methods_by_name['GetMaterializationJob']._serialized_options = b'\202\323\344\223\002%\" /v1/jobs/get-materialization-job:\001*\242\274\346\300\005\002@\001'
  _MATERIALIZATIONJOBSERVICE.methods_by_name['CancelMaterializationJob']._options = None
  _MATERIALIZATIONJOBSERVICE.methods_by_name['CancelMaterializationJob']._serialized_options = b'\202\323\344\223\002(\"#/v1/jobs/cancel-materialization-job:\001*\242\274\346\300\005\002@\001'
  _MATERIALIZATIONJOBSERVICE.methods_by_name['GetLatestReadyTime']._options = None
  _MATERIALIZATIONJOBSERVICE.methods_by_name['GetLatestReadyTime']._serialized_options = b'\202\323\344\223\002#\"\036/v1/jobs/get-latest-ready-time:\001*\242\274\346\300\005<\032\r\010\001\022\tworkspace*+tecton/workspace#read_materialization_state'
  _MATERIALIZATIONJOBSERVICE.methods_by_name['TestOnlyGetMaterializationTaskParams']._options = None
  _MATERIALIZATIONJOBSERVICE.methods_by_name['TestOnlyGetMaterializationTaskParams']._serialized_options = b'\242\274\346\300\005\033*\031tecton/organization#admin'
  _MATERIALIZATIONJOBREQUEST._serialized_start=290
  _MATERIALIZATIONJOBREQUEST._serialized_end=637
  _MATERIALIZATIONJOBRESPONSE._serialized_start=639
  _MATERIALIZATIONJOBRESPONSE._serialized_end=745
  _MATERIALIZATIONJOB._serialized_start=748
  _MATERIALIZATIONJOB._serialized_end=1328
  _JOBATTEMPT._serialized_start=1331
  _JOBATTEMPT._serialized_end=1524
  _LISTJOBSREQUEST._serialized_start=1526
  _LISTJOBSREQUEST._serialized_end=1608
  _LISTJOBSRESPONSE._serialized_start=1610
  _LISTJOBSRESPONSE._serialized_end=1708
  _GETJOBREQUEST._serialized_start=1710
  _GETJOBREQUEST._serialized_end=1813
  _GETJOBRESPONSE._serialized_start=1815
  _GETJOBRESPONSE._serialized_end=1909
  _CANCELJOBREQUEST._serialized_start=1911
  _CANCELJOBREQUEST._serialized_end=2017
  _CANCELJOBRESPONSE._serialized_start=2019
  _CANCELJOBRESPONSE._serialized_end=2116
  _GETLATESTREADYTIMEREQUEST._serialized_start=2119
  _GETLATESTREADYTIMEREQUEST._serialized_end=2252
  _GETLATESTREADYTIMERESPONSE._serialized_start=2255
  _GETLATESTREADYTIMERESPONSE._serialized_end=2455
  _TESTONLYGETMATERIALIZATIONTASKPARAMSREQUEST._serialized_start=2457
  _TESTONLYGETMATERIALIZATIONTASKPARAMSREQUEST._serialized_end=2576
  _TESTONLYGETMATERIALIZATIONTASKPARAMSRESPONSE._serialized_start=2578
  _TESTONLYGETMATERIALIZATIONTASKPARAMSRESPONSE._serialized_end=2694
  _MATERIALIZATIONJOBSERVICE._serialized_start=2697
  _MATERIALIZATIONJOBSERVICE._serialized_end=4166
# @@protoc_insertion_point(module_scope)
