# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/databricks_api/libraries.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tecton_proto.databricks_api import jobs_pb2 as tecton__proto_dot_databricks__api_dot_jobs__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n+tecton_proto/databricks_api/libraries.proto\x12\x1btecton_proto.databricks_api\x1a&tecton_proto/databricks_api/jobs.proto\"\x80\x01\n\x15LibraryInstallRequest\x12\x1d\n\ncluster_id\x18\x01 \x01(\tR\tclusterId\x12H\n\tlibraries\x18\x02 \x03(\x0b\x32*.tecton_proto.databricks_api.RemoteLibraryR\tlibraries\"s\n\x15\x43lusterStatusResponse\x12Z\n\x10library_statuses\x18\x01 \x03(\x0b\x32/.tecton_proto.databricks_api.LibraryStatusEntryR\x0flibraryStatuses\"Z\n\x12LibraryStatusEntry\x12\x44\n\x07library\x18\x01 \x01(\x0b\x32*.tecton_proto.databricks_api.RemoteLibraryR\x07libraryB\x1d\n\x19\x63om.tecton.databricks_apiP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.databricks_api.libraries_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\031com.tecton.databricks_apiP\001'
  _LIBRARYINSTALLREQUEST._serialized_start=117
  _LIBRARYINSTALLREQUEST._serialized_end=245
  _CLUSTERSTATUSRESPONSE._serialized_start=247
  _CLUSTERSTATUSRESPONSE._serialized_end=362
  _LIBRARYSTATUSENTRY._serialized_start=364
  _LIBRARYSTATUSENTRY._serialized_end=454
# @@protoc_insertion_point(module_scope)
