# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/data/internal_spark_cluster_status.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n5tecton_proto/data/internal_spark_cluster_status.proto\x12\x11tecton_proto.data\"\xc3\x01\n\x1aInternalSparkClusterStatus\x12I\n\x06status\x18\x01 \x01(\x0e\x32\x31.tecton_proto.data.InternalSparkClusterStatusEnumR\x06status\x12\x14\n\x05\x65rror\x18\x02 \x01(\x08R\x05\x65rror\x12#\n\rerror_message\x18\x03 \x01(\tR\x0c\x65rrorMessage\x12\x1f\n\x0b\x63luster_url\x18\x04 \x01(\tR\nclusterUrl*\xc5\x02\n\x1eInternalSparkClusterStatusEnum\x12)\n%INTERNAL_SPARK_CLUSTER_STATUS_UNKNOWN\x10\x00\x12,\n(INTERNAL_SPARK_CLUSTER_STATUS_NO_CLUSTER\x10\x01\x12\x32\n.INTERNAL_SPARK_CLUSTER_STATUS_CREATING_CLUSTER\x10\x02\x12>\n:INTERNAL_SPARK_CLUSTER_STATUS_WAITING_FOR_CLUSTER_TO_START\x10\x03\x12)\n%INTERNAL_SPARK_CLUSTER_STATUS_HEALTHY\x10\x04\x12+\n\'INTERNAL_SPARK_CLUSTER_STATUS_UNHEALTHY\x10\x05\x42\x13\n\x0f\x63om.tecton.dataP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.data.internal_spark_cluster_status_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\017com.tecton.dataP\001'
  _INTERNALSPARKCLUSTERSTATUSENUM._serialized_start=275
  _INTERNALSPARKCLUSTERSTATUSENUM._serialized_end=600
  _INTERNALSPARKCLUSTERSTATUS._serialized_start=77
  _INTERNALSPARKCLUSTERSTATUS._serialized_end=272
# @@protoc_insertion_point(module_scope)
