# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/online_store/status_entry.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n,tecton_proto/online_store/status_entry.proto\x12\x19tecton_proto.online_store\"z\n\x0bStatusEntry\x12\x1f\n\x0bsource_type\x18\x01 \x01(\tR\nsourceType\x12)\n\x11raw_data_end_time\x18\x02 \x01(\x03R\x0erawDataEndTime\x12\x1f\n\x0b\x61nchor_time\x18\x03 \x01(\x03R\nanchorTimeB\x1a\n\x16\x63om.tecton.onlinestoreP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.online_store.status_entry_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\026com.tecton.onlinestoreP\001'
  _STATUSENTRY._serialized_start=75
  _STATUSENTRY._serialized_end=197
# @@protoc_insertion_point(module_scope)
