# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/databricks_api/workspace.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n+tecton_proto/databricks_api/workspace.proto\x12\x1btecton_proto.databricks_api\"*\n\x14ListWorkspaceRequest\x12\x12\n\x04path\x18\x01 \x01(\tR\x04path\"Z\n\x15ListWorkspaceResponse\x12\x41\n\x07objects\x18\x01 \x03(\x0b\x32\'.tecton_proto.databricks_api.ObjectInfoR\x07objects\"z\n\nObjectInfo\x12\x1f\n\x0bobject_type\x18\x01 \x01(\tR\nobjectType\x12\x1b\n\tobject_id\x18\x02 \x01(\x03R\x08objectId\x12\x12\n\x04path\x18\x03 \x01(\tR\x04path\x12\x1a\n\x08language\x18\x04 \x01(\tR\x08languageB\x1d\n\x19\x63om.tecton.databricks_apiP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.databricks_api.workspace_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\031com.tecton.databricks_apiP\001'
  _LISTWORKSPACEREQUEST._serialized_start=76
  _LISTWORKSPACEREQUEST._serialized_end=118
  _LISTWORKSPACERESPONSE._serialized_start=120
  _LISTWORKSPACERESPONSE._serialized_end=210
  _OBJECTINFO._serialized_start=212
  _OBJECTINFO._serialized_end=334
# @@protoc_insertion_point(module_scope)
