# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/common/fco_locator.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tecton_proto.common import id_pb2 as tecton__proto_dot_common_dot_id__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n%tecton_proto/common/fco_locator.proto\x12\x13tecton_proto.common\x1a\x1ctecton_proto/common/id.proto\"v\n\nFcoLocator\x12\x14\n\x04name\x18\x01 \x01(\tH\x00R\x04name\x12)\n\x02id\x18\x02 \x01(\x0b\x32\x17.tecton_proto.common.IdH\x00R\x02id\x12\x1c\n\tworkspace\x18\x03 \x01(\tR\tworkspaceB\t\n\x07locator\"[\n\x0cIdFcoLocator\x12\'\n\x02id\x18\x02 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x02id\x12\x1c\n\tworkspace\x18\x03 \x01(\tR\tworkspaceJ\x04\x08\x01\x10\x02\x42\x15\n\x11\x63om.tecton.commonP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.common.fco_locator_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\021com.tecton.commonP\001'
  _FCOLOCATOR._serialized_start=92
  _FCOLOCATOR._serialized_end=210
  _IDFCOLOCATOR._serialized_start=212
  _IDFCOLOCATOR._serialized_end=303
# @@protoc_insertion_point(module_scope)
