# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/common/schema_container.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tecton_proto.common import schema_pb2 as tecton__proto_dot_common_dot_schema__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n*tecton_proto/common/schema_container.proto\x12\x13tecton_proto.common\x1a tecton_proto/common/schema.proto\"g\n\x0fSchemaContainer\x12\x42\n\rtecton_schema\x18\x01 \x01(\x0b\x32\x1b.tecton_proto.common.SchemaH\x00R\x0ctectonSchemaB\x10\n\x0eschema_optionsB\x15\n\x11\x63om.tecton.commonP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.common.schema_container_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\021com.tecton.commonP\001'
  _SCHEMACONTAINER._serialized_start=101
  _SCHEMACONTAINER._serialized_end=204
# @@protoc_insertion_point(module_scope)
