# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/data/feature_view.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from tecton_proto.args import pipeline_pb2 as tecton__proto_dot_args_dot_pipeline__pb2
from tecton_proto.args import feature_view_pb2 as tecton__proto_dot_args_dot_feature__view__pb2
from tecton_proto.common import aggregation_function_pb2 as tecton__proto_dot_common_dot_aggregation__function__pb2
from tecton_proto.data import fv_materialization_pb2 as tecton__proto_dot_data_dot_fv__materialization__pb2
from tecton_proto.common import schema_pb2 as tecton__proto_dot_common_dot_schema__pb2
from tecton_proto.common import data_source_type_pb2 as tecton__proto_dot_common_dot_data__source__type__pb2
from tecton_proto.common import framework_version_pb2 as tecton__proto_dot_common_dot_framework__version__pb2
from tecton_proto.data import fco_metadata_pb2 as tecton__proto_dot_data_dot_fco__metadata__pb2
from tecton_proto.common import id_pb2 as tecton__proto_dot_common_dot_id__pb2
from tecton_proto.validation import validator_pb2 as tecton__proto_dot_validation_dot_validator__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n$tecton_proto/data/feature_view.proto\x12\x11tecton_proto.data\x1a\x1egoogle/protobuf/duration.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a tecton_proto/args/pipeline.proto\x1a$tecton_proto/args/feature_view.proto\x1a.tecton_proto/common/aggregation_function.proto\x1a*tecton_proto/data/fv_materialization.proto\x1a tecton_proto/common/schema.proto\x1a*tecton_proto/common/data_source_type.proto\x1a+tecton_proto/common/framework_version.proto\x1a$tecton_proto/data/fco_metadata.proto\x1a\x1ctecton_proto/common/id.proto\x1a\'tecton_proto/validation/validator.proto\"\xb1\r\n\x0b\x46\x65\x61tureView\x12?\n\x0f\x66\x65\x61ture_view_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\rfeatureViewId\x12\x41\n\x0c\x66\x63o_metadata\x18\x02 \x01(\x0b\x32\x1e.tecton_proto.data.FcoMetadataR\x0b\x66\x63oMetadata\x12\x36\n\nentity_ids\x18\x03 \x03(\x0b\x32\x17.tecton_proto.common.IdR\tentityIds\x12\x1b\n\tjoin_keys\x18\x04 \x03(\tR\x08joinKeys\x12?\n\x07schemas\x18\x0b \x01(\x0b\x32%.tecton_proto.data.FeatureViewSchemasR\x07schemas\x12P\n\x0b\x65nrichments\x18\xe8\x07 \x01(\x0b\x32).tecton_proto.data.FeatureViewEnrichmentsB\x02\x18\x01R\x0b\x65nrichments\x12U\n\x12temporal_aggregate\x18\x07 \x01(\x0b\x32$.tecton_proto.data.TemporalAggregateH\x00R\x11temporalAggregate\x12\x39\n\x08temporal\x18\x08 \x01(\x0b\x32\x1b.tecton_proto.data.TemporalH\x00R\x08temporal\x12]\n\x16on_demand_feature_view\x18\x11 \x01(\x0b\x32&.tecton_proto.data.OnDemandFeatureViewH\x00R\x13onDemandFeatureView\x12\x46\n\rfeature_table\x18\x10 \x01(\x0b\x32\x1f.tecton_proto.data.FeatureTableH\x00R\x0c\x66\x65\x61tureTable\x12#\n\rtimestamp_key\x18\n \x01(\tR\x0ctimestampKey\x12W\n\x14online_serving_index\x18\x05 \x01(\x0b\x32%.tecton_proto.data.OnlineServingIndexR\x12onlineServingIndex\x12\x37\n\x08pipeline\x18\x06 \x01(\x0b\x32\x1b.tecton_proto.args.PipelineR\x08pipeline\x12\x62\n\x16materialization_params\x18\t \x01(\x0b\x32+.tecton_proto.data.NewMaterializationParamsR\x15materializationParams\x12\x37\n\x17materialization_enabled\x18\x0c \x01(\x08R\x16materializationEnabled\x12}\n!materialization_state_transitions\x18\r \x03(\x0b\x32\x31.tecton_proto.data.MaterializationStateTransitionR\x1fmaterializationStateTransitions\x12P\n\x11monitoring_params\x18\x0e \x01(\x0b\x32#.tecton_proto.data.MonitoringParamsR\x10monitoringParams\x12?\n\x1c\x66\x65\x61ture_store_format_version\x18\x17 \x01(\x05R\x19\x66\x65\x61tureStoreFormatVersion\x12G\n\x0esnowflake_data\x18\x18 \x01(\x0b\x32 .tecton_proto.data.SnowflakeDataR\rsnowflakeData\x12/\n\x11\x66ramework_version\x18\x13 \x01(\x05\x42\x02\x18\x01R\x10\x66rameworkVersion\x12H\n\nfw_version\x18\x1a \x01(\x0e\x32%.tecton_proto.common.FrameworkVersionB\x02\x18\x01R\tfwVersion\x12\x17\n\x07web_url\x18\x19 \x01(\tR\x06webUrl\x12H\n\rbatch_trigger\x18\x1b \x01(\x0e\x32#.tecton_proto.args.BatchTriggerTypeR\x0c\x62\x61tchTrigger\x12[\n\x0fvalidation_args\x18\x1c \x01(\x0b\x32\x32.tecton_proto.validation.FeatureViewValidationArgsR\x0evalidationArgsB\x13\n\x11\x66\x65\x61ture_view_type\"\xbe\x02\n\x11TemporalAggregate\x12@\n\x0eslide_interval\x18\x01 \x01(\x0b\x32\x19.google.protobuf.DurationR\rslideInterval\x12\x32\n\x15slide_interval_string\x18\x05 \x01(\tR\x13slideIntervalString\x12?\n\x08\x66\x65\x61tures\x18\x02 \x03(\x0b\x32#.tecton_proto.data.AggregateFeatureR\x08\x66\x65\x61tures\x12#\n\ris_continuous\x18\x03 \x01(\x08R\x0cisContinuous\x12M\n\x10\x64\x61ta_source_type\x18\x04 \x01(\x0e\x32#.tecton_proto.common.DataSourceTypeR\x0e\x64\x61taSourceType\"\xc2\x02\n\x10\x41ggregateFeature\x12,\n\x12input_feature_name\x18\x01 \x01(\tR\x10inputFeatureName\x12.\n\x13output_feature_name\x18\x02 \x01(\tR\x11outputFeatureName\x12\x44\n\x08\x66unction\x18\x03 \x01(\x0e\x32(.tecton_proto.common.AggregationFunctionR\x08\x66unction\x12W\n\x0f\x66unction_params\x18\x05 \x01(\x0b\x32..tecton_proto.common.AggregationFunctionParamsR\x0e\x66unctionParams\x12\x31\n\x06window\x18\x04 \x01(\x0b\x32\x19.google.protobuf.DurationR\x06window\"\xf5\x01\n\x1dTrailingTimeWindowAggregation\x12\x19\n\x08time_key\x18\x01 \x01(\tR\x07timeKey\x12S\n\x18\x61ggregation_slide_period\x18\x02 \x01(\x0b\x32\x19.google.protobuf.DurationR\x16\x61ggregationSlidePeriod\x12?\n\x08\x66\x65\x61tures\x18\x03 \x03(\x0b\x32#.tecton_proto.data.AggregateFeatureR\x08\x66\x65\x61tures\x12#\n\ris_continuous\x18\x04 \x01(\x08R\x0cisContinuous\"\xbb\x02\n\x08Temporal\x12:\n\x0bserving_ttl\x18\x10 \x01(\x0b\x32\x19.google.protobuf.DurationR\nservingTtl\x12M\n\x10\x64\x61ta_source_type\x18\x11 \x01(\x0e\x32#.tecton_proto.common.DataSourceTypeR\x0e\x64\x61taSourceType\x12J\n\x0f\x62\x61\x63kfill_config\x18\x12 \x01(\x0b\x32!.tecton_proto.args.BackfillConfigR\x0e\x62\x61\x63kfillConfig\x12\x33\n\x15incremental_backfills\x18\x13 \x01(\x08R\x14incrementalBackfills\x12#\n\ris_continuous\x18\x14 \x01(\x08R\x0cisContinuous\"\x9a\x01\n\x0c\x46\x65\x61tureTable\x12%\n\x0eonline_enabled\x18\x01 \x01(\x08R\ronlineEnabled\x12\'\n\x0foffline_enabled\x18\x02 \x01(\x08R\x0eofflineEnabled\x12:\n\x0bserving_ttl\x18\x03 \x01(\x0b\x32\x19.google.protobuf.DurationR\nservingTtl\"*\n\x13OnDemandFeatureView\x12\x13\n\x05no_op\x18\x01 \x01(\x08R\x04noOp\"\xa6\x01\n\x12\x46\x65\x61tureViewSchemas\x12<\n\x0bview_schema\x18\x01 \x01(\x0b\x32\x1b.tecton_proto.common.SchemaR\nviewSchema\x12R\n\x16materialization_schema\x18\x02 \x01(\x0b\x32\x1b.tecton_proto.common.SchemaR\x15materializationSchema\"1\n\x12OnlineServingIndex\x12\x1b\n\tjoin_keys\x18\x01 \x03(\tR\x08joinKeys\"\xc4\x02\n\x1eMaterializationStateTransition\x12\x38\n\ttimestamp\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\ttimestamp\x12%\n\x0eonline_enabled\x18\x02 \x01(\x08R\ronlineEnabled\x12\'\n\x0foffline_enabled\x18\x03 \x01(\x08R\x0eofflineEnabled\x12R\n\x17\x66\x65\x61ture_start_timestamp\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x15\x66\x65\x61tureStartTimestamp\x12\x44\n\x1ematerialization_serial_version\x18\x05 \x01(\x05R\x1cmaterializationSerialVersion\"\xaf\x01\n\x19ParquetOfflineStoreParams\x12I\n\x13time_partition_size\x18\x01 \x01(\x0b\x32\x19.google.protobuf.DurationR\x11timePartitionSize\x12G\n\x07version\x18\x02 \x01(\x0e\x32-.tecton_proto.data.ParquetOfflineStoreVersionR\x07version\"\xab\x01\n\x17\x44\x65ltaOfflineStoreParams\x12I\n\x13time_partition_size\x18\x01 \x01(\x0b\x32\x19.google.protobuf.DurationR\x11timePartitionSize\x12\x45\n\x07version\x18\x02 \x01(\x0e\x32+.tecton_proto.data.DeltaOfflineStoreVersionR\x07version\"\xac\x01\n\x12OfflineStoreParams\x12H\n\x07parquet\x18\x01 \x01(\x0b\x32,.tecton_proto.data.ParquetOfflineStoreParamsH\x00R\x07parquet\x12\x42\n\x05\x64\x65lta\x18\x02 \x01(\x0b\x32*.tecton_proto.data.DeltaOfflineStoreParamsH\x00R\x05\x64\x65ltaB\x08\n\x06params\"\x98\t\n\x18NewMaterializationParams\x12\x46\n\x11schedule_interval\x18\x01 \x01(\x0b\x32\x19.google.protobuf.DurationR\x10scheduleInterval\x12\x62\n\x1fmaterialization_start_timestamp\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x1dmaterializationStartTimestamp\x12R\n\x17\x66\x65\x61ture_start_timestamp\x18\r \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x15\x66\x65\x61tureStartTimestamp\x12^\n\x1emax_batch_aggregation_interval\x18\x03 \x01(\x0b\x32\x19.google.protobuf.DurationR\x1bmaxBatchAggregationInterval\x12\x33\n\x16writes_to_online_store\x18\x04 \x01(\x08R\x13writesToOnlineStore\x12\x35\n\x17writes_to_offline_store\x18\x05 \x01(\x08R\x14writesToOfflineStore\x12^\n\x14offline_store_config\x18\x06 \x01(\x0b\x32,.tecton_proto.args.OfflineFeatureStoreConfigR\x12offlineStoreConfig\x12W\n\x14offline_store_params\x18\x0e \x01(\x0b\x32%.tecton_proto.data.OfflineStoreParamsR\x12offlineStoreParams\x12U\n\x15\x62\x61tch_materialization\x18\x07 \x01(\x0b\x32 .tecton_proto.args.ClusterConfigR\x14\x62\x61tchMaterialization\x12W\n\x16stream_materialization\x18\x08 \x01(\x0b\x32 .tecton_proto.args.ClusterConfigR\x15streamMaterialization\x12L\n\x15max_source_data_delay\x18\t \x01(\x0b\x32\x19.google.protobuf.DurationR\x12maxSourceDataDelay\x12T\n\x13online_store_params\x18\n \x01(\x0b\x32$.tecton_proto.data.OnlineStoreParamsR\x11onlineStoreParams\x12\x44\n\routput_stream\x18\x0b \x01(\x0b\x32\x1f.tecton_proto.args.OutputStreamR\x0coutputStream\x12]\n\x11time_range_policy\x18\x0c \x01(\x0e\x32\x31.tecton_proto.data.MaterializationTimeRangePolicyR\x0ftimeRangePolicy\"\xc7\x01\n\x11OnlineStoreParams\x12%\n\x0euser_specified\x18\x01 \x01(\x08R\ruserSpecified\x12@\n\x06\x64ynamo\x18\x02 \x01(\x0b\x32&.tecton_proto.args.DynamoDbOnlineStoreH\x00R\x06\x64ynamo\x12;\n\x05redis\x18\x03 \x01(\x0b\x32#.tecton_proto.args.RedisOnlineStoreH\x00R\x05redisB\x0c\n\nstore_type\"\x92\x02\n\x10MonitoringParams\x12%\n\x0euser_specified\x18\x01 \x01(\x08R\ruserSpecified\x12+\n\x11monitor_freshness\x18\x02 \x01(\x08R\x10monitorFreshness\x12W\n\x1a\x65xpected_feature_freshness\x18\x03 \x01(\x0b\x32\x19.google.protobuf.DurationR\x18\x65xpectedFeatureFreshness\x12\x1f\n\x0b\x61lert_email\x18\x04 \x01(\tR\nalertEmail\x12\x30\n\x14grace_period_seconds\x18\x05 \x01(\x05R\x12gracePeriodSeconds\"\x85\x01\n\x16\x46\x65\x61tureViewEnrichments\x12S\n\x12\x66p_materialization\x18\x04 \x01(\x0b\x32$.tecton_proto.data.FvMaterializationR\x11\x66pMaterializationJ\x04\x08\x01\x10\x02J\x04\x08\x02\x10\x03J\x04\x08\x03\x10\x04J\x04\x08\x05\x10\x06\"?\n\rSnowflakeData\x12.\n\x13snowflake_view_name\x18\x01 \x01(\tR\x11snowflakeViewName*\x91\x01\n\x1aParquetOfflineStoreVersion\x12)\n%PARQUET_OFFLINE_STORE_VERSION_UNKNOWN\x10\x00\x12#\n\x1fPARQUET_OFFLINE_STORE_VERSION_1\x10\x01\x12#\n\x1fPARQUET_OFFLINE_STORE_VERSION_2\x10\x02*f\n\x18\x44\x65ltaOfflineStoreVersion\x12\'\n#DELTA_OFFLINE_STORE_VERSION_UNKNOWN\x10\x00\x12!\n\x1d\x44\x45LTA_OFFLINE_STORE_VERSION_1\x10\x01*\xc6\x01\n\x1eMaterializationTimeRangePolicy\x12\x31\n-MATERIALIZATION_TIME_RANGE_POLICY_UNSPECIFIED\x10\x00\x12:\n6MATERIALIZATION_TIME_RANGE_POLICY_FAIL_IF_OUT_OF_RANGE\x10\x01\x12\x35\n1MATERIALIZATION_TIME_RANGE_POLICY_FILTER_TO_RANGE\x10\x02\x42\x13\n\x0f\x63om.tecton.dataP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.data.feature_view_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\017com.tecton.dataP\001'
  _FEATUREVIEW.fields_by_name['enrichments']._options = None
  _FEATUREVIEW.fields_by_name['enrichments']._serialized_options = b'\030\001'
  _FEATUREVIEW.fields_by_name['framework_version']._options = None
  _FEATUREVIEW.fields_by_name['framework_version']._serialized_options = b'\030\001'
  _FEATUREVIEW.fields_by_name['fw_version']._options = None
  _FEATUREVIEW.fields_by_name['fw_version']._serialized_options = b'\030\001'
  _PARQUETOFFLINESTOREVERSION._serialized_start=6583
  _PARQUETOFFLINESTOREVERSION._serialized_end=6728
  _DELTAOFFLINESTOREVERSION._serialized_start=6730
  _DELTAOFFLINESTOREVERSION._serialized_end=6832
  _MATERIALIZATIONTIMERANGEPOLICY._serialized_start=6835
  _MATERIALIZATIONTIMERANGEPOLICY._serialized_end=7033
  _FEATUREVIEW._serialized_start=521
  _FEATUREVIEW._serialized_end=2234
  _TEMPORALAGGREGATE._serialized_start=2237
  _TEMPORALAGGREGATE._serialized_end=2555
  _AGGREGATEFEATURE._serialized_start=2558
  _AGGREGATEFEATURE._serialized_end=2880
  _TRAILINGTIMEWINDOWAGGREGATION._serialized_start=2883
  _TRAILINGTIMEWINDOWAGGREGATION._serialized_end=3128
  _TEMPORAL._serialized_start=3131
  _TEMPORAL._serialized_end=3446
  _FEATURETABLE._serialized_start=3449
  _FEATURETABLE._serialized_end=3603
  _ONDEMANDFEATUREVIEW._serialized_start=3605
  _ONDEMANDFEATUREVIEW._serialized_end=3647
  _FEATUREVIEWSCHEMAS._serialized_start=3650
  _FEATUREVIEWSCHEMAS._serialized_end=3816
  _ONLINESERVINGINDEX._serialized_start=3818
  _ONLINESERVINGINDEX._serialized_end=3867
  _MATERIALIZATIONSTATETRANSITION._serialized_start=3870
  _MATERIALIZATIONSTATETRANSITION._serialized_end=4194
  _PARQUETOFFLINESTOREPARAMS._serialized_start=4197
  _PARQUETOFFLINESTOREPARAMS._serialized_end=4372
  _DELTAOFFLINESTOREPARAMS._serialized_start=4375
  _DELTAOFFLINESTOREPARAMS._serialized_end=4546
  _OFFLINESTOREPARAMS._serialized_start=4549
  _OFFLINESTOREPARAMS._serialized_end=4721
  _NEWMATERIALIZATIONPARAMS._serialized_start=4724
  _NEWMATERIALIZATIONPARAMS._serialized_end=5900
  _ONLINESTOREPARAMS._serialized_start=5903
  _ONLINESTOREPARAMS._serialized_end=6102
  _MONITORINGPARAMS._serialized_start=6105
  _MONITORINGPARAMS._serialized_end=6379
  _FEATUREVIEWENRICHMENTS._serialized_start=6382
  _FEATUREVIEWENRICHMENTS._serialized_end=6515
  _SNOWFLAKEDATA._serialized_start=6517
  _SNOWFLAKEDATA._serialized_end=6580
# @@protoc_insertion_point(module_scope)
