# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/databricks_api/execution.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n+tecton_proto/databricks_api/execution.proto\x12\x1btecton_proto.databricks_api\"Q\n\x15\x43ontextsCreateRequest\x12\x1a\n\x08language\x18\x01 \x01(\tR\x08language\x12\x1c\n\tclusterId\x18\x02 \x01(\tR\tclusterId\"(\n\x16\x43ontextsCreateResponse\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\"R\n\x14\x43ontextStatusRequest\x12\x1c\n\tclusterId\x18\x01 \x01(\tR\tclusterId\x12\x1c\n\tcontextId\x18\x02 \x01(\tR\tcontextId\"U\n\x15\x43ontextStatusResponse\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\x12\x16\n\x06status\x18\x02 \x01(\tR\x06status\x12\x14\n\x05\x65rror\x18\x03 \x01(\tR\x05\x65rror\"\x8a\x01\n\x16\x43ommandsExecuteRequest\x12\x1a\n\x08language\x18\x01 \x01(\tR\x08language\x12\x1c\n\tclusterId\x18\x02 \x01(\tR\tclusterId\x12\x1c\n\tcontextId\x18\x03 \x01(\tR\tcontextId\x12\x18\n\x07\x63ommand\x18\x04 \x01(\tR\x07\x63ommand\")\n\x17\x43ommandsExecuteResponse\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02id\"q\n\x15\x43ommandsStatusRequest\x12\x1c\n\tclusterId\x18\x01 \x01(\tR\tclusterId\x12\x1c\n\tcontextId\x18\x02 \x01(\tR\tcontextId\x12\x1c\n\tcommandId\x18\x03 \x01(\tR\tcommandId\"S\n\x07Results\x12\x12\n\x04\x64\x61ta\x18\x01 \x01(\tR\x04\x64\x61ta\x12\x1e\n\nresultType\x18\x02 \x01(\tR\nresultType\x12\x14\n\x05\x63\x61use\x18\x03 \x01(\tR\x05\x63\x61use\"p\n\x16\x43ommandsStatusResponse\x12>\n\x07results\x18\x01 \x01(\x0b\x32$.tecton_proto.databricks_api.ResultsR\x07results\x12\x16\n\x06status\x18\x02 \x01(\tR\x06status\"S\n\x15\x43ontextDestroyRequest\x12\x1c\n\tclusterId\x18\x01 \x01(\tR\tclusterId\x12\x1c\n\tcontextId\x18\x02 \x01(\tR\tcontextId\"(\n\x16\x43ontextDestroyResponse\x12\x0e\n\x02id\x18\x01 \x01(\tR\x02idB\x1d\n\x19\x63om.tecton.databricks_apiP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.databricks_api.execution_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\031com.tecton.databricks_apiP\001'
  _CONTEXTSCREATEREQUEST._serialized_start=76
  _CONTEXTSCREATEREQUEST._serialized_end=157
  _CONTEXTSCREATERESPONSE._serialized_start=159
  _CONTEXTSCREATERESPONSE._serialized_end=199
  _CONTEXTSTATUSREQUEST._serialized_start=201
  _CONTEXTSTATUSREQUEST._serialized_end=283
  _CONTEXTSTATUSRESPONSE._serialized_start=285
  _CONTEXTSTATUSRESPONSE._serialized_end=370
  _COMMANDSEXECUTEREQUEST._serialized_start=373
  _COMMANDSEXECUTEREQUEST._serialized_end=511
  _COMMANDSEXECUTERESPONSE._serialized_start=513
  _COMMANDSEXECUTERESPONSE._serialized_end=554
  _COMMANDSSTATUSREQUEST._serialized_start=556
  _COMMANDSSTATUSREQUEST._serialized_end=669
  _RESULTS._serialized_start=671
  _RESULTS._serialized_end=754
  _COMMANDSSTATUSRESPONSE._serialized_start=756
  _COMMANDSSTATUSRESPONSE._serialized_end=868
  _CONTEXTDESTROYREQUEST._serialized_start=870
  _CONTEXTDESTROYREQUEST._serialized_end=953
  _CONTEXTDESTROYRESPONSE._serialized_start=955
  _CONTEXTDESTROYRESPONSE._serialized_end=995
# @@protoc_insertion_point(module_scope)
