# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/common/column_type.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n%tecton_proto/common/column_type.proto\x12\x13tecton_proto.common*\xab\x02\n\nColumnType\x12\x17\n\x13\x43OLUMN_TYPE_UNKNOWN\x10\x00\x12\x15\n\x11\x43OLUMN_TYPE_INT64\x10\x01\x12\x16\n\x12\x43OLUMN_TYPE_DOUBLE\x10\x02\x12\x16\n\x12\x43OLUMN_TYPE_STRING\x10\x03\x12\x14\n\x10\x43OLUMN_TYPE_BOOL\x10\x06\x12\x1c\n\x18\x43OLUMN_TYPE_STRING_ARRAY\x10\x07\x12\x1b\n\x17\x43OLUMN_TYPE_INT64_ARRAY\x10\x08\x12\x1c\n\x18\x43OLUMN_TYPE_DOUBLE_ARRAY\x10\t\x12\x1b\n\x17\x43OLUMN_TYPE_FLOAT_ARRAY\x10\n\x12%\n!COLUMN_TYPE_DERIVE_FROM_DATA_TYPE\x10\x0b\"\x04\x08\x04\x10\x04\"\x04\x08\x05\x10\x05\x42?\n\x11\x63om.tecton.commonP\x01Z(github.com/tecton-ai/tecton_proto/common')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.common.column_type_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\021com.tecton.commonP\001Z(github.com/tecton-ai/tecton_proto/common'
  _COLUMNTYPE._serialized_start=63
  _COLUMNTYPE._serialized_end=362
# @@protoc_insertion_point(module_scope)
