# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/spark_api/jobs.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tecton_proto.spark_common import clusters_pb2 as tecton__proto_dot_spark__common_dot_clusters__pb2
from tecton_proto.spark_common import libraries_pb2 as tecton__proto_dot_spark__common_dot_libraries__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n!tecton_proto/spark_api/jobs.proto\x12\x16tecton_proto.spark_api\x1a(tecton_proto/spark_common/clusters.proto\x1a)tecton_proto/spark_common/libraries.proto\"\xb7\x03\n\x19PythonMaterializationTask\x12\x38\n\x18materialization_path_uri\x18\x01 \x01(\tR\x16materializationPathUri\x12n\n\x0f\x62\x61se_parameters\x18\x02 \x03(\x0b\x32\x45.tecton_proto.spark_api.PythonMaterializationTask.BaseParametersEntryR\x0e\x62\x61seParameters\x12V\n\x08taskType\x18\x03 \x01(\x0e\x32:.tecton_proto.spark_api.PythonMaterializationTask.TaskTypeR\x08taskType\x1a\x41\n\x13\x42\x61seParametersEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\"U\n\x08TaskType\x12\r\n\tSTREAMING\x10\x00\x12\t\n\x05\x42\x41TCH\x10\x01\x12\n\n\x06INGEST\x10\x02\x12\x0c\n\x08\x44\x45LETION\x10\x03\x12\x15\n\x11\x44\x45LTA_MAINTENANCE\x10\x04\"\x98\x04\n\x0fStartJobRequest\x12H\n\x0bnew_cluster\x18\x01 \x01(\x0b\x32%.tecton_proto.spark_common.NewClusterH\x00R\nnewCluster\x12W\n\x10\x65xisting_cluster\x18\x07 \x01(\x0b\x32*.tecton_proto.spark_common.ExistingClusterH\x00R\x0f\x65xistingCluster\x12\x64\n\x14materialization_task\x18\x02 \x01(\x0b\x32\x31.tecton_proto.spark_api.PythonMaterializationTaskR\x13materializationTask\x12\x19\n\x08run_name\x18\x03 \x01(\tR\x07runName\x12@\n\tlibraries\x18\x04 \x03(\x0b\x32\".tecton_proto.spark_common.LibraryR\tlibraries\x12\'\n\x0ftimeout_seconds\x18\x05 \x01(\x05R\x0etimeoutSeconds\x12\x1f\n\x0bis_notebook\x18\x06 \x01(\x08R\nisNotebook\x12>\n\x1buse_stepped_materialization\x18\x0b \x01(\x08R\x19useSteppedMaterializationB\t\n\x07\x63lusterJ\x04\x08\t\x10\nJ\x04\x08\n\x10\x0b\")\n\x10StartJobResponse\x12\x15\n\x06run_id\x18\x01 \x01(\tR\x05runId\"&\n\rGetJobRequest\x12\x15\n\x06run_id\x18\x01 \x01(\tR\x05runId\"\x86\x03\n\x0eGetJobResponse\x12\x15\n\x06run_id\x18\x01 \x01(\tR\x05runId\x12\x15\n\x06job_id\x18\x07 \x01(\tR\x05jobId\x12 \n\x0crun_page_url\x18\x03 \x01(\tR\nrunPageUrl\x12(\n\x10spark_cluster_id\x18\x06 \x01(\tR\x0esparkClusterId\x12<\n\x07\x64\x65tails\x18\x04 \x01(\x0b\x32\".tecton_proto.spark_api.RunDetailsR\x07\x64\x65tails\x12o\n\x13\x61\x64\x64itional_metadata\x18\x05 \x03(\x0b\x32>.tecton_proto.spark_api.GetJobResponse.AdditionalMetadataEntryR\x12\x61\x64\x64itionalMetadata\x1a\x45\n\x17\x41\x64\x64itionalMetadataEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01J\x04\x08\x02\x10\x03\"\xd0\x01\n\nRunDetails\x12@\n\nrun_status\x18\x01 \x01(\x0e\x32!.tecton_proto.spark_api.RunStatusR\trunStatus\x12[\n\x12termination_reason\x18\x03 \x01(\x0e\x32,.tecton_proto.spark_api.RunTerminationReasonR\x11terminationReason\x12#\n\rstate_message\x18\x02 \x01(\tR\x0cstateMessage\"\'\n\x0eStopJobRequest\x12\x15\n\x06run_id\x18\x01 \x01(\tR\x05runId\"@\n\x0eListJobRequest\x12\x16\n\x06offset\x18\x01 \x01(\x05R\x06offset\x12\x16\n\x06marker\x18\x02 \x01(\tR\x06marker\"\x9f\x02\n\nRunSummary\x12\x15\n\x06run_id\x18\x01 \x01(\tR\x05runId\x12\x1b\n\trun_state\x18\x02 \x01(\tR\x08runState\x12)\n\x10resource_locator\x18\x03 \x01(\tR\x0fresourceLocator\x12k\n\x13\x61\x64\x64itional_metadata\x18\x04 \x03(\x0b\x32:.tecton_proto.spark_api.RunSummary.AdditionalMetadataEntryR\x12\x61\x64\x64itionalMetadata\x1a\x45\n\x17\x41\x64\x64itionalMetadataEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\"|\n\x0fListJobResponse\x12\x36\n\x04runs\x18\x01 \x03(\x0b\x32\".tecton_proto.spark_api.RunSummaryR\x04runs\x12\x19\n\x08has_more\x18\x02 \x01(\x08R\x07hasMore\x12\x16\n\x06marker\x18\x03 \x01(\tR\x06marker*\xb6\x01\n\tRunStatus\x12\x16\n\x12RUN_STATUS_UNKNOWN\x10\x00\x12\x16\n\x12RUN_STATUS_PENDING\x10\x01\x12\x16\n\x12RUN_STATUS_RUNNING\x10\x02\x12\x16\n\x12RUN_STATUS_SUCCESS\x10\x03\x12\x14\n\x10RUN_STATUS_ERROR\x10\x04\x12\x1a\n\x16RUN_STATUS_TERMINATING\x10\x05\x12\x17\n\x13RUN_STATUS_CANCELED\x10\x06*\x98\x01\n\x14RunTerminationReason\x12\x1e\n\x1aUNKNOWN_TERMINATION_REASON\x10\x00\x12\x10\n\x0cJOB_FINISHED\x10\x01\x12\x16\n\x12MANUAL_CANCELATION\x10\x02\x12\x1f\n\x1bINSTANCE_ALLOCATION_FAILURE\x10\x03\x12\x15\n\x11NON_CLOUD_FAILURE\x10\x04\x42\x18\n\x14\x63om.tecton.spark_apiP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.spark_api.jobs_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\024com.tecton.spark_apiP\001'
  _PYTHONMATERIALIZATIONTASK_BASEPARAMETERSENTRY._options = None
  _PYTHONMATERIALIZATIONTASK_BASEPARAMETERSENTRY._serialized_options = b'8\001'
  _GETJOBRESPONSE_ADDITIONALMETADATAENTRY._options = None
  _GETJOBRESPONSE_ADDITIONALMETADATAENTRY._serialized_options = b'8\001'
  _RUNSUMMARY_ADDITIONALMETADATAENTRY._options = None
  _RUNSUMMARY_ADDITIONALMETADATAENTRY._serialized_options = b'8\001'
  _RUNSTATUS._serialized_start=2338
  _RUNSTATUS._serialized_end=2520
  _RUNTERMINATIONREASON._serialized_start=2523
  _RUNTERMINATIONREASON._serialized_end=2675
  _PYTHONMATERIALIZATIONTASK._serialized_start=147
  _PYTHONMATERIALIZATIONTASK._serialized_end=586
  _PYTHONMATERIALIZATIONTASK_BASEPARAMETERSENTRY._serialized_start=434
  _PYTHONMATERIALIZATIONTASK_BASEPARAMETERSENTRY._serialized_end=499
  _PYTHONMATERIALIZATIONTASK_TASKTYPE._serialized_start=501
  _PYTHONMATERIALIZATIONTASK_TASKTYPE._serialized_end=586
  _STARTJOBREQUEST._serialized_start=589
  _STARTJOBREQUEST._serialized_end=1125
  _STARTJOBRESPONSE._serialized_start=1127
  _STARTJOBRESPONSE._serialized_end=1168
  _GETJOBREQUEST._serialized_start=1170
  _GETJOBREQUEST._serialized_end=1208
  _GETJOBRESPONSE._serialized_start=1211
  _GETJOBRESPONSE._serialized_end=1601
  _GETJOBRESPONSE_ADDITIONALMETADATAENTRY._serialized_start=1526
  _GETJOBRESPONSE_ADDITIONALMETADATAENTRY._serialized_end=1595
  _RUNDETAILS._serialized_start=1604
  _RUNDETAILS._serialized_end=1812
  _STOPJOBREQUEST._serialized_start=1814
  _STOPJOBREQUEST._serialized_end=1853
  _LISTJOBREQUEST._serialized_start=1855
  _LISTJOBREQUEST._serialized_end=1919
  _RUNSUMMARY._serialized_start=1922
  _RUNSUMMARY._serialized_end=2209
  _RUNSUMMARY_ADDITIONALMETADATAENTRY._serialized_start=1526
  _RUNSUMMARY_ADDITIONALMETADATAENTRY._serialized_end=1595
  _LISTJOBRESPONSE._serialized_start=2211
  _LISTJOBRESPONSE._serialized_end=2335
# @@protoc_insertion_point(module_scope)
