# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/args/entity.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tecton_proto.args import basic_info_pb2 as tecton__proto_dot_args_dot_basic__info__pb2
from tecton_proto.common import id_pb2 as tecton__proto_dot_common_dot_id__pb2
from tecton_proto.args import diff_options_pb2 as tecton__proto_dot_args_dot_diff__options__pb2
from tecton_proto.common import framework_version_pb2 as tecton__proto_dot_common_dot_framework__version__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1etecton_proto/args/entity.proto\x12\x11tecton_proto.args\x1a\"tecton_proto/args/basic_info.proto\x1a\x1ctecton_proto/common/id.proto\x1a$tecton_proto/args/diff_options.proto\x1a+tecton_proto/common/framework_version.proto\"\x90\x02\n\nEntityArgs\x12\x34\n\tentity_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x08\x65ntityId\x12\x37\n\x04info\x18\x02 \x01(\x0b\x32\x1c.tecton_proto.args.BasicInfoB\x05\x92M\x02\x10\x01R\x04info\x12\x46\n\x07version\x18\x04 \x01(\x0e\x32%.tecton_proto.common.FrameworkVersionB\x05\x92M\x02\x08\x05R\x07version\x12.\n\x0fprevent_destroy\x18\x05 \x01(\x08\x42\x05\x92M\x02\x08\x01R\x0epreventDestroy\x12\x1b\n\tjoin_keys\x18\x03 \x03(\tR\x08joinKeysB\x13\n\x0f\x63om.tecton.argsP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.args.entity_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\017com.tecton.argsP\001'
  _ENTITYARGS.fields_by_name['info']._options = None
  _ENTITYARGS.fields_by_name['info']._serialized_options = b'\222M\002\020\001'
  _ENTITYARGS.fields_by_name['version']._options = None
  _ENTITYARGS.fields_by_name['version']._serialized_options = b'\222M\002\010\005'
  _ENTITYARGS.fields_by_name['prevent_destroy']._options = None
  _ENTITYARGS.fields_by_name['prevent_destroy']._serialized_options = b'\222M\002\010\001'
  _ENTITYARGS._serialized_start=203
  _ENTITYARGS._serialized_end=475
# @@protoc_insertion_point(module_scope)
