# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/args/diff_options.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import descriptor_pb2 as google_dot_protobuf_dot_descriptor__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n$tecton_proto/args/diff_options.proto\x12\x11tecton_proto.args\x1a google/protobuf/descriptor.proto\"\xb9\x01\n\x0b\x44iffOptions\x12\x39\n\x06update\x18\x01 \x01(\x0e\x32!.tecton_proto.args.UpdateStrategyR\x06update\x12\x1b\n\thide_path\x18\x02 \x01(\x08R\x08hidePath\x12R\n\x0erendering_type\x18\x03 \x01(\x0e\x32+.tecton_proto.args.FcoPropertyRenderingTypeR\rrenderingType*\xe9\x01\n\x0eUpdateStrategy\x12\x0c\n\x08RECREATE\x10\x00\x12\x0b\n\x07INPLACE\x10\x01\x12\x12\n\x0eINPLACE_ON_ADD\x10\x03\x12\x15\n\x11INPLACE_ON_REMOVE\x10\x04\x12\x0b\n\x07PASSIVE\x10\x05\x12\x1e\n\x1aRECREATE_UNLESS_SUPPRESSED\x10\x06\x12\x35\n1RECREATE_UNLESS_SUPPRESSED_INVALIDATE_CHECKPOINTS\x10\x07\x12-\n)RECREATE_UNLESS_SUPPRESSED_RESTART_STREAM\x10\x08*\x97\x02\n\x18\x46\x63oPropertyRenderingType\x12+\n\'FCO_PROPERTY_RENDERING_TYPE_UNSPECIFIED\x10\x00\x12*\n&FCO_PROPERTY_RENDERING_TYPE_PLAIN_TEXT\x10\x01\x12&\n\"FCO_PROPERTY_RENDERING_TYPE_PYTHON\x10\x02\x12#\n\x1f\x46\x43O_PROPERTY_RENDERING_TYPE_SQL\x10\x03\x12-\n)FCO_PROPERTY_RENDERING_TYPE_ONLY_DECLARED\x10\x04\x12&\n\"FCO_PROPERTY_RENDERING_TYPE_HIDDEN\x10\x05:a\n\x0c\x64iff_options\x12\x1d.google.protobuf.FieldOptions\x18\xd2\t \x01(\x0b\x32\x1e.tecton_proto.args.DiffOptionsR\x0b\x64iffOptionsB\x13\n\x0f\x63om.tecton.argsP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.args.diff_options_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:
  google_dot_protobuf_dot_descriptor__pb2.FieldOptions.RegisterExtension(diff_options)

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\017com.tecton.argsP\001'
  _UPDATESTRATEGY._serialized_start=282
  _UPDATESTRATEGY._serialized_end=515
  _FCOPROPERTYRENDERINGTYPE._serialized_start=518
  _FCOPROPERTYRENDERINGTYPE._serialized_end=797
  _DIFFOPTIONS._serialized_start=94
  _DIFFOPTIONS._serialized_end=279
# @@protoc_insertion_point(module_scope)
