# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/databricks_api/jobs.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tecton_proto.spark_common import clusters_pb2 as tecton__proto_dot_spark__common_dot_clusters__pb2
from tecton_proto.spark_common import libraries_pb2 as tecton__proto_dot_spark__common_dot_libraries__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n&tecton_proto/databricks_api/jobs.proto\x12\x1btecton_proto.databricks_api\x1a(tecton_proto/spark_common/clusters.proto\x1a)tecton_proto/spark_common/libraries.proto\"\xde\x01\n\x0cNotebookTask\x12#\n\rnotebook_path\x18\x01 \x01(\tR\x0cnotebookPath\x12\x66\n\x0f\x62\x61se_parameters\x18\x02 \x03(\x0b\x32=.tecton_proto.databricks_api.NotebookTask.BaseParametersEntryR\x0e\x62\x61seParameters\x1a\x41\n\x13\x42\x61seParametersEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\"\xda\x01\n\rRemoteLibrary\x12\x12\n\x03jar\x18\x01 \x01(\tH\x00R\x03jar\x12\x12\n\x03\x65gg\x18\x02 \x01(\tH\x00R\x03\x65gg\x12\x12\n\x03whl\x18\x03 \x01(\tH\x00R\x03whl\x12?\n\x05maven\x18\x04 \x01(\x0b\x32\'.tecton_proto.spark_common.MavenLibraryH\x00R\x05maven\x12<\n\x04pypi\x18\x05 \x01(\x0b\x32&.tecton_proto.spark_common.PyPiLibraryH\x00R\x04pypiB\x0e\n\x0clibrary_type\"\xcf\x03\n\x04Task\x12\x19\n\x08task_key\x18\x01 \x01(\tR\x07taskKey\x12 \n\x0b\x64\x65scription\x18\x02 \x01(\tR\x0b\x64\x65scription\x12@\n\ndepends_on\x18\x03 \x03(\x0b\x32!.tecton_proto.databricks_api.TaskR\tdependsOn\x12H\n\x0bnew_cluster\x18\x04 \x01(\x0b\x32%.tecton_proto.spark_common.NewClusterH\x00R\nnewCluster\x12\x30\n\x13\x65xisting_cluster_id\x18\x05 \x01(\tH\x00R\x11\x65xistingClusterId\x12N\n\rnotebook_task\x18\x06 \x01(\x0b\x32).tecton_proto.databricks_api.NotebookTaskR\x0cnotebookTask\x12H\n\tlibraries\x18\x07 \x03(\x0b\x32*.tecton_proto.databricks_api.RemoteLibraryR\tlibraries\x12\'\n\x0ftimeout_seconds\x18\x08 \x01(\x05R\x0etimeoutSecondsB\t\n\x07\x63luster\"\xb5\x03\n\x15JobsRunsSubmitRequest\x12H\n\x0bnew_cluster\x18\x01 \x01(\x0b\x32%.tecton_proto.spark_common.NewClusterH\x00R\nnewCluster\x12\x30\n\x13\x65xisting_cluster_id\x18\x06 \x01(\tH\x00R\x11\x65xistingClusterId\x12N\n\rnotebook_task\x18\x02 \x01(\x0b\x32).tecton_proto.databricks_api.NotebookTaskR\x0cnotebookTask\x12\x19\n\x08run_name\x18\x03 \x01(\tR\x07runName\x12H\n\tlibraries\x18\x04 \x03(\x0b\x32*.tecton_proto.databricks_api.RemoteLibraryR\tlibraries\x12\'\n\x0ftimeout_seconds\x18\x05 \x01(\x05R\x0etimeoutSeconds\x12\x37\n\x05tasks\x18\x07 \x03(\x0b\x32!.tecton_proto.databricks_api.TaskR\x05tasksB\t\n\x07\x63luster\"/\n\x16JobsRunsSubmitResponse\x12\x15\n\x06run_id\x18\x01 \x01(\x03R\x05runId\"+\n\x12JobsRunsGetRequest\x12\x15\n\x06run_id\x18\x01 \x01(\x03R\x05runId\"\x81\x02\n\x13JobsRunsGetResponse\x12\x15\n\x06run_id\x18\x01 \x01(\x03R\x05runId\x12\x15\n\x06job_id\x18\x02 \x01(\x03R\x05jobId\x12W\n\x10\x63luster_instance\x18\x05 \x01(\x0b\x32,.tecton_proto.databricks_api.ClusterInstanceR\x0f\x63lusterInstance\x12 \n\x0crun_page_url\x18\x03 \x01(\tR\nrunPageUrl\x12;\n\x05state\x18\x04 \x01(\x0b\x32%.tecton_proto.databricks_api.RunStateR\x05stateJ\x04\x08\x06\x10\x07\"0\n\x0f\x43lusterInstance\x12\x1d\n\ncluster_id\x18\x01 \x01(\tR\tclusterId\"\xe4\x03\n\x08RunState\x12\x61\n\x10life_cycle_state\x18\x01 \x01(\x0e\x32\x37.tecton_proto.databricks_api.RunState.RunLifeCycleStateR\x0elifeCycleState\x12W\n\x0cresult_state\x18\x02 \x01(\x0e\x32\x34.tecton_proto.databricks_api.RunState.RunResultStateR\x0bresultState\x12#\n\rstate_message\x18\x03 \x01(\tR\x0cstateMessage\"\x91\x01\n\x11RunLifeCycleState\x12 \n\x1cUNKNOWN_RUN_LIFE_CYCLE_STATE\x10\x00\x12\x0b\n\x07PENDING\x10\x01\x12\x0b\n\x07RUNNING\x10\x02\x12\x0f\n\x0bTERMINATING\x10\x03\x12\x0e\n\nTERMINATED\x10\x04\x12\x0b\n\x07SKIPPED\x10\x05\x12\x12\n\x0eINTERNAL_ERROR\x10\x06\"c\n\x0eRunResultState\x12\x1c\n\x18UNKNOWN_RUN_RESULT_STATE\x10\x00\x12\x0b\n\x07SUCCESS\x10\x01\x12\n\n\x06\x46\x41ILED\x10\x02\x12\x0c\n\x08TIMEDOUT\x10\x03\x12\x0c\n\x08\x43\x41NCELED\x10\x04\"-\n\x14JobsCancelRunRequest\x12\x15\n\x06run_id\x18\x01 \x01(\x03R\x05runId\"\x7f\n\x13JobsRunsListRequest\x12\x16\n\x06offset\x18\x01 \x01(\x05R\x06offset\x12\x1f\n\x0b\x61\x63tive_only\x18\x02 \x01(\x08R\nactiveOnly\x12\x19\n\x08run_type\x18\x03 \x01(\tR\x07runType\x12\x14\n\x05limit\x18\x04 \x01(\x05R\x05limit\"\xa5\x01\n\nNewCluster\x12X\n\x0b\x63ustom_tags\x18\x01 \x03(\x0b\x32\x37.tecton_proto.databricks_api.NewCluster.CustomTagsEntryR\ncustomTags\x1a=\n\x0f\x43ustomTagsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\"\xe0\x01\n\x0b\x43lusterSpec\x12J\n\x0bnew_cluster\x18\x01 \x01(\x0b\x32\'.tecton_proto.databricks_api.NewClusterH\x00R\nnewCluster\x12\x30\n\x13\x65xisting_cluster_id\x18\x03 \x01(\tH\x00R\x11\x65xistingClusterId\x12H\n\tlibraries\x18\x02 \x03(\x0b\x32*.tecton_proto.databricks_api.RemoteLibraryR\tlibrariesB\t\n\x07\x63luster\"\xdf\x01\n\x03Run\x12\x15\n\x06job_id\x18\x01 \x01(\x03R\x05jobId\x12\x15\n\x06run_id\x18\x02 \x01(\x03R\x05runId\x12;\n\x05state\x18\x03 \x01(\x0b\x32%.tecton_proto.databricks_api.RunStateR\x05state\x12K\n\x0c\x63luster_spec\x18\x04 \x01(\x0b\x32(.tecton_proto.databricks_api.ClusterSpecR\x0b\x63lusterSpec\x12 \n\x0crun_page_url\x18\x05 \x01(\tR\nrunPageUrl\"g\n\x14JobsRunsListResponse\x12\x34\n\x04runs\x18\x01 \x03(\x0b\x32 .tecton_proto.databricks_api.RunR\x04runs\x12\x19\n\x08has_more\x18\x02 \x01(\x08R\x07hasMoreB\x1d\n\x19\x63om.tecton.databricks_apiP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.databricks_api.jobs_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\031com.tecton.databricks_apiP\001'
  _NOTEBOOKTASK_BASEPARAMETERSENTRY._options = None
  _NOTEBOOKTASK_BASEPARAMETERSENTRY._serialized_options = b'8\001'
  _NEWCLUSTER_CUSTOMTAGSENTRY._options = None
  _NEWCLUSTER_CUSTOMTAGSENTRY._serialized_options = b'8\001'
  _NOTEBOOKTASK._serialized_start=157
  _NOTEBOOKTASK._serialized_end=379
  _NOTEBOOKTASK_BASEPARAMETERSENTRY._serialized_start=314
  _NOTEBOOKTASK_BASEPARAMETERSENTRY._serialized_end=379
  _REMOTELIBRARY._serialized_start=382
  _REMOTELIBRARY._serialized_end=600
  _TASK._serialized_start=603
  _TASK._serialized_end=1066
  _JOBSRUNSSUBMITREQUEST._serialized_start=1069
  _JOBSRUNSSUBMITREQUEST._serialized_end=1506
  _JOBSRUNSSUBMITRESPONSE._serialized_start=1508
  _JOBSRUNSSUBMITRESPONSE._serialized_end=1555
  _JOBSRUNSGETREQUEST._serialized_start=1557
  _JOBSRUNSGETREQUEST._serialized_end=1600
  _JOBSRUNSGETRESPONSE._serialized_start=1603
  _JOBSRUNSGETRESPONSE._serialized_end=1860
  _CLUSTERINSTANCE._serialized_start=1862
  _CLUSTERINSTANCE._serialized_end=1910
  _RUNSTATE._serialized_start=1913
  _RUNSTATE._serialized_end=2397
  _RUNSTATE_RUNLIFECYCLESTATE._serialized_start=2151
  _RUNSTATE_RUNLIFECYCLESTATE._serialized_end=2296
  _RUNSTATE_RUNRESULTSTATE._serialized_start=2298
  _RUNSTATE_RUNRESULTSTATE._serialized_end=2397
  _JOBSCANCELRUNREQUEST._serialized_start=2399
  _JOBSCANCELRUNREQUEST._serialized_end=2444
  _JOBSRUNSLISTREQUEST._serialized_start=2446
  _JOBSRUNSLISTREQUEST._serialized_end=2573
  _NEWCLUSTER._serialized_start=2576
  _NEWCLUSTER._serialized_end=2741
  _NEWCLUSTER_CUSTOMTAGSENTRY._serialized_start=2680
  _NEWCLUSTER_CUSTOMTAGSENTRY._serialized_end=2741
  _CLUSTERSPEC._serialized_start=2744
  _CLUSTERSPEC._serialized_end=2968
  _RUN._serialized_start=2971
  _RUN._serialized_end=3194
  _JOBSRUNSLISTRESPONSE._serialized_start=3196
  _JOBSRUNSLISTRESPONSE._serialized_end=3299
# @@protoc_insertion_point(module_scope)
