# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/common/secret.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n tecton_proto/common/secret.proto\x12\x13tecton_proto.common\"\x84\x01\n\x06Secret\x12\x16\n\x05value\x18\x01 \x01(\tH\x00R\x05value\x12\'\n\x0eredacted_value\x18\x02 \x01(\tH\x00R\rredactedValue\x12)\n\x0f\x65ncrypted_value\x18\x03 \x01(\tH\x00R\x0e\x65ncryptedValueB\x0e\n\x0csecret_valueB\x15\n\x11\x63om.tecton.commonP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.common.secret_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\021com.tecton.commonP\001'
  _SECRET._serialized_start=58
  _SECRET._serialized_end=190
# @@protoc_insertion_point(module_scope)
