# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/data/fco.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tecton_proto.common import id_pb2 as tecton__proto_dot_common_dot_id__pb2
from tecton_proto.data import entity_pb2 as tecton__proto_dot_data_dot_entity__pb2
from tecton_proto.data import virtual_data_source_pb2 as tecton__proto_dot_data_dot_virtual__data__source__pb2
from tecton_proto.data import feature_view_pb2 as tecton__proto_dot_data_dot_feature__view__pb2
from tecton_proto.data import feature_service_pb2 as tecton__proto_dot_data_dot_feature__service__pb2
from tecton_proto.data import transformation_pb2 as tecton__proto_dot_data_dot_transformation__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1btecton_proto/data/fco.proto\x12\x11tecton_proto.data\x1a\x1ctecton_proto/common/id.proto\x1a\x1etecton_proto/data/entity.proto\x1a+tecton_proto/data/virtual_data_source.proto\x1a$tecton_proto/data/feature_view.proto\x1a\'tecton_proto/data/feature_service.proto\x1a&tecton_proto/data/transformation.proto\"\x85\x03\n\x03\x46\x63o\x12V\n\x13virtual_data_source\x18\x01 \x01(\x0b\x32$.tecton_proto.data.VirtualDataSourceH\x00R\x11virtualDataSource\x12\x33\n\x06\x65ntity\x18\x02 \x01(\x0b\x32\x19.tecton_proto.data.EntityH\x00R\x06\x65ntity\x12\x43\n\x0c\x66\x65\x61ture_view\x18\x06 \x01(\x0b\x32\x1e.tecton_proto.data.FeatureViewH\x00R\x0b\x66\x65\x61tureView\x12L\n\x0f\x66\x65\x61ture_service\x18\x04 \x01(\x0b\x32!.tecton_proto.data.FeatureServiceH\x00R\x0e\x66\x65\x61tureService\x12K\n\x0etransformation\x18\x07 \x01(\x0b\x32!.tecton_proto.data.TransformationH\x00R\x0etransformationB\x05\n\x03\x66\x63oJ\x04\x08\x03\x10\x04J\x04\x08\x05\x10\x06\"n\n\x0c\x46\x63oContainer\x12\x32\n\x08root_ids\x18\x01 \x03(\x0b\x32\x17.tecton_proto.common.IdR\x07rootIds\x12*\n\x04\x66\x63os\x18\x02 \x03(\x0b\x32\x16.tecton_proto.data.FcoR\x04\x66\x63osB\x13\n\x0f\x63om.tecton.dataP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.data.fco_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\017com.tecton.dataP\001'
  _FCO._serialized_start=277
  _FCO._serialized_end=666
  _FCOCONTAINER._serialized_start=668
  _FCOCONTAINER._serialized_end=778
# @@protoc_insertion_point(module_scope)
