# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/data/fv_materialization.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tecton_proto.common import fco_locator_pb2 as tecton__proto_dot_common_dot_fco__locator__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n*tecton_proto/data/fv_materialization.proto\x12\x11tecton_proto.data\x1a%tecton_proto/common/fco_locator.proto\"\"\n\x0c\x46ileLocation\x12\x12\n\x04path\x18\x01 \x01(\tR\x04path\"\x80\x04\n\x11\x46vMaterialization\x12X\n\x17id_feature_view_locator\x18\t \x01(\x0b\x32!.tecton_proto.common.IdFcoLocatorR\x14idFeatureViewLocator\x12\x44\n\x1ematerialization_serial_version\x18\x03 \x01(\x05R\x1cmaterializationSerialVersion\x12]\n\x1amaterialized_data_location\x18\x05 \x01(\x0b\x32\x1f.tecton_proto.data.FileLocationR\x18materializedDataLocation\x12\x65\n\x1estreaming_checkpoint_locations\x18\x06 \x03(\x0b\x32\x1f.tecton_proto.data.FileLocationR\x1cstreamingCheckpointLocations\x12\x32\n\x15has_materialized_data\x18\x07 \x01(\x08R\x13hasMaterializedData\x12?\n\x1c\x66\x65\x61ture_store_format_version\x18\x08 \x01(\x05R\x19\x66\x65\x61tureStoreFormatVersionJ\x04\x08\x01\x10\x02J\x04\x08\x02\x10\x03J\x04\x08\x04\x10\x05\x42\x13\n\x0f\x63om.tecton.dataP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.data.fv_materialization_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\017com.tecton.dataP\001'
  _FILELOCATION._serialized_start=104
  _FILELOCATION._serialized_end=138
  _FVMATERIALIZATION._serialized_start=141
  _FVMATERIALIZATION._serialized_end=653
# @@protoc_insertion_point(module_scope)
