# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/args/version_constraints.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import descriptor_pb2 as google_dot_protobuf_dot_descriptor__pb2
from tecton_proto.common import framework_version_pb2 as tecton__proto_dot_common_dot_framework__version__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n+tecton_proto/args/version_constraints.proto\x12\x11tecton_proto.args\x1a google/protobuf/descriptor.proto\x1a+tecton_proto/common/framework_version.proto\"\xad\x01\n\x1b\x46rameworkVersionConstraints\x12\x46\n\x0bmin_version\x18\x01 \x01(\x0e\x32%.tecton_proto.common.FrameworkVersionR\nminVersion\x12\x46\n\x0bmax_version\x18\x02 \x01(\x0e\x32%.tecton_proto.common.FrameworkVersionR\nmaxVersion:{\n\x11\x66ramework_version\x12\x1d.google.protobuf.FieldOptions\x18\xd0\x0f \x01(\x0b\x32..tecton_proto.args.FrameworkVersionConstraintsR\x10\x66rameworkVersionB\x13\n\x0f\x63om.tecton.argsP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.args.version_constraints_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:
  google_dot_protobuf_dot_descriptor__pb2.FieldOptions.RegisterExtension(framework_version)

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\017com.tecton.argsP\001'
  _FRAMEWORKVERSIONCONSTRAINTS._serialized_start=146
  _FRAMEWORKVERSIONCONSTRAINTS._serialized_end=319
# @@protoc_insertion_point(module_scope)
