# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/args/pipeline.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from tecton_proto.common import id_pb2 as tecton__proto_dot_common_dot_id__pb2
from tecton_proto.common import spark_schema_pb2 as tecton__proto_dot_common_dot_spark__schema__pb2
from tecton_proto.args import feature_service_pb2 as tecton__proto_dot_args_dot_feature__service__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n tecton_proto/args/pipeline.proto\x12\x11tecton_proto.args\x1a\x1egoogle/protobuf/duration.proto\x1a\x1bgoogle/protobuf/empty.proto\x1a\x1ctecton_proto/common/id.proto\x1a&tecton_proto/common/spark_schema.proto\x1a\'tecton_proto/args/feature_service.proto\"z\n\x08Pipeline\x12\x33\n\x04root\x18\x01 \x01(\x0b\x32\x1f.tecton_proto.args.PipelineNodeR\x04root\x12\x33\n\x04mode\x18\x02 \x01(\x0e\x32\x1f.tecton_proto.args.PipelineModeR\x04modeJ\x04\x08\x03\x10\x04\"\xbc\x04\n\x0cPipelineNode\x12X\n\x13transformation_node\x18\x01 \x01(\x0b\x32%.tecton_proto.args.TransformationNodeH\x00R\x12transformationNode\x12M\n\x10\x64\x61ta_source_node\x18\x02 \x01(\x0b\x32!.tecton_proto.args.DataSourceNodeH\x00R\x0e\x64\x61taSourceNode\x12\x46\n\rconstant_node\x18\x03 \x01(\x0b\x32\x1f.tecton_proto.args.ConstantNodeH\x00R\x0c\x63onstantNode\x12\x63\n\x18request_data_source_node\x18\x04 \x01(\x0b\x32(.tecton_proto.args.RequestDataSourceNodeH\x00R\x15requestDataSourceNode\x12P\n\x11\x66\x65\x61ture_view_node\x18\x05 \x01(\x0b\x32\".tecton_proto.args.FeatureViewNodeH\x00R\x0f\x66\x65\x61tureViewNode\x12q\n\x1cmaterialization_context_node\x18\x07 \x01(\x0b\x32-.tecton_proto.args.MaterializationContextNodeH\x00R\x1amaterializationContextNodeB\x0b\n\tnode_typeJ\x04\x08\x06\x10\x07\"\x82\x01\n\x15RequestDataSourceNode\x12J\n\x0frequest_context\x18\x01 \x01(\x0b\x32!.tecton_proto.args.RequestContextR\x0erequestContext\x12\x1d\n\ninput_name\x18\x02 \x01(\tR\tinputName\"J\n\x0eRequestContext\x12\x38\n\x06schema\x18\x01 \x01(\x0b\x32 .tecton_proto.common.SparkSchemaR\x06schema\"\xc5\x01\n\x0f\x46\x65\x61tureViewNode\x12?\n\x0f\x66\x65\x61ture_view_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\rfeatureViewId\x12R\n\x0c\x66\x65\x61ture_view\x18\x02 \x01(\x0b\x32/.tecton_proto.args.FeatureServiceFeaturePackageR\x0b\x66\x65\x61tureView\x12\x1d\n\ninput_name\x18\x03 \x01(\tR\tinputName\"\x8c\x01\n\x12TransformationNode\x12\x44\n\x11transformation_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x10transformationId\x12\x30\n\x06inputs\x18\x02 \x03(\x0b\x32\x18.tecton_proto.args.InputR\x06inputs\"\x84\x01\n\x05Input\x12\x1d\n\targ_index\x18\x01 \x01(\x05H\x00R\x08\x61rgIndex\x12\x1b\n\x08\x61rg_name\x18\x02 \x01(\tH\x00R\x07\x61rgName\x12\x33\n\x04node\x18\x03 \x01(\x0b\x32\x1f.tecton_proto.args.PipelineNodeR\x04nodeB\n\n\x08\x61rg_type\"\xc4\x03\n\x0e\x44\x61taSourceNode\x12L\n\x16virtual_data_source_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x13virtualDataSourceId\x12\x33\n\x06window\x18\x02 \x01(\x0b\x32\x19.google.protobuf.DurationH\x00R\x06window\x12>\n\x1awindow_unbounded_preceding\x18\x04 \x01(\x08H\x00R\x18windowUnboundedPreceding\x12+\n\x10window_unbounded\x18\x06 \x01(\x08H\x00R\x0fwindowUnbounded\x12G\n\x11start_time_offset\x18\x07 \x01(\x0b\x32\x19.google.protobuf.DurationH\x00R\x0fstartTimeOffset\x12\x42\n\x0fschedule_offset\x18\x03 \x01(\x0b\x32\x19.google.protobuf.DurationR\x0escheduleOffset\x12\x1d\n\ninput_name\x18\x05 \x01(\tR\tinputNameB\x16\n\x14\x64\x61ta_window_override\"\xe0\x01\n\x0c\x43onstantNode\x12#\n\x0cstring_const\x18\x01 \x01(\tH\x00R\x0bstringConst\x12\x1d\n\tint_const\x18\x02 \x01(\tH\x00R\x08intConst\x12!\n\x0b\x66loat_const\x18\x03 \x01(\tH\x00R\nfloatConst\x12\x1f\n\nbool_const\x18\x04 \x01(\x08H\x00R\tboolConst\x12\x37\n\nnull_const\x18\x05 \x01(\x0b\x32\x16.google.protobuf.EmptyH\x00R\tnullConstB\x0f\n\rconstant_type\"\x1c\n\x1aMaterializationContextNode*\x9a\x01\n\x0cPipelineMode\x12\x19\n\x15PIPELINE_MODE_UNKNOWN\x10\x00\x12\x17\n\x13PIPELINE_MODE_SPARK\x10\x01\x12\x1b\n\x17PIPELINE_MODE_ON_DEMAND\x10\x02\x12\x1f\n\x1bPIPELINE_MODE_SNOWFLAKE_SQL\x10\x03\x12\x18\n\x14PIPELINE_MODE_ATHENA\x10\x04\x42\x13\n\x0f\x63om.tecton.argsP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.args.pipeline_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\017com.tecton.argsP\001'
  _PIPELINEMODE._serialized_start=2326
  _PIPELINEMODE._serialized_end=2480
  _PIPELINE._serialized_start=227
  _PIPELINE._serialized_end=349
  _PIPELINENODE._serialized_start=352
  _PIPELINENODE._serialized_end=924
  _REQUESTDATASOURCENODE._serialized_start=927
  _REQUESTDATASOURCENODE._serialized_end=1057
  _REQUESTCONTEXT._serialized_start=1059
  _REQUESTCONTEXT._serialized_end=1133
  _FEATUREVIEWNODE._serialized_start=1136
  _FEATUREVIEWNODE._serialized_end=1333
  _TRANSFORMATIONNODE._serialized_start=1336
  _TRANSFORMATIONNODE._serialized_end=1476
  _INPUT._serialized_start=1479
  _INPUT._serialized_end=1611
  _DATASOURCENODE._serialized_start=1614
  _DATASOURCENODE._serialized_end=2066
  _CONSTANTNODE._serialized_start=2069
  _CONSTANTNODE._serialized_end=2293
  _MATERIALIZATIONCONTEXTNODE._serialized_start=2295
  _MATERIALIZATIONCONTEXTNODE._serialized_end=2323
# @@protoc_insertion_point(module_scope)
