DEFAULT_AGGREGATION_TILES_WINDOW_START_COLUMN_NAME = "tile_start_time"
DEFAULT_AGGREGATION_TILES_WINDOW_END_COLUMN_NAME = "tile_end_time"

AGGREGATION_LEVEL_DISABLED = "disabled"
AGGREGATION_LEVEL_PARTIAL = "partial"
AGGREGATION_LEVEL_FULL = "full"
SUPPORTED_AGGREGATION_LEVEL_VALUES = [
    AGGREGATION_LEVEL_DISABLED,
    AGGREGATION_LEVEL_PARTIAL,
    AGGREGATION_LEVEL_FULL,
]
