# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/data/hive_metastore.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n&tecton_proto/data/hive_metastore.proto\x12\x11tecton_proto.data\"G\n\x13HiveTableDataSource\x12\x14\n\x05table\x18\x01 \x01(\tR\x05table\x12\x1a\n\x08\x64\x61tabase\x18\x02 \x01(\tR\x08\x64\x61tabase\"&\n\x0eListHiveResult\x12\x14\n\x05names\x18\x01 \x03(\tR\x05namesB\x13\n\x0f\x63om.tecton.dataP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.data.hive_metastore_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\017com.tecton.dataP\001'
  _HIVETABLEDATASOURCE._serialized_start=61
  _HIVETABLEDATASOURCE._serialized_end=132
  _LISTHIVERESULT._serialized_start=134
  _LISTHIVERESULT._serialized_end=172
# @@protoc_insertion_point(module_scope)
