# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/dataobs/validation_task_params.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from tecton_proto.common import id_pb2 as tecton__proto_dot_common_dot_id__pb2
from tecton_proto.data import feature_view_pb2 as tecton__proto_dot_data_dot_feature__view__pb2
from tecton_proto.data import user_deployment_settings_pb2 as tecton__proto_dot_data_dot_user__deployment__settings__pb2
from tecton_proto.dataobs import expectation_pb2 as tecton__proto_dot_dataobs_dot_expectation__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n1tecton_proto/dataobs/validation_task_params.proto\x12\x14tecton_proto.dataobs\x1a\x1egoogle/protobuf/duration.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1ctecton_proto/common/id.proto\x1a$tecton_proto/data/feature_view.proto\x1a\x30tecton_proto/data/user_deployment_settings.proto\x1a&tecton_proto/dataobs/expectation.proto\"\xd2\x04\n\x14ValidationTaskParams\x12\x43\n\x11validation_job_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x0fvalidationJobId\x12\x17\n\x07task_id\x18\x02 \x01(\x05R\x06taskId\x12%\n\x0eworkspace_name\x18\x03 \x01(\tR\rworkspaceName\x12\x41\n\x0c\x66\x65\x61ture_view\x18\x04 \x01(\x0b\x32\x1e.tecton_proto.data.FeatureViewR\x0b\x66\x65\x61tureView\x12X\n\x13metric_expectations\x18\x05 \x03(\x0b\x32\'.tecton_proto.dataobs.MetricExpectationR\x12metricExpectations\x12V\n\x12\x64ynamo_data_source\x18\x08 \x01(\x0b\x32&.tecton_proto.dataobs.DynamoDataSourceH\x00R\x10\x64ynamoDataSource\x12J\n\x0es3_data_source\x18\t \x01(\x0b\x32\".tecton_proto.dataobs.S3DataSourceH\x00R\x0cs3DataSource\x12\x46\n\x0fresult_location\x18\x06 \x01(\x0b\x32\x1d.tecton_proto.data.S3LocationR\x0eresultLocation\x12\x1d\n\nresult_key\x18\x07 \x01(\tR\tresultKeyB\r\n\x0b\x64\x61ta_source\"\xc5\x02\n\x10\x44ynamoDataSource\x12>\n\x0b\x64ynamo_role\x18\x01 \x01(\x0b\x32\x1d.tecton_proto.data.AwsIamRoleR\ndynamoRole\x12\x1d\n\ntable_name\x18\x02 \x01(\tR\ttableName\x12\x42\n\x0fmetric_interval\x18\x03 \x01(\x0b\x32\x19.google.protobuf.DurationR\x0emetricInterval\x12H\n\x12\x66\x65\x61ture_start_time\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x10\x66\x65\x61tureStartTime\x12\x44\n\x10\x66\x65\x61ture_end_time\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x0e\x66\x65\x61tureEndTime\"*\n\x0cS3DataSource\x12\x1a\n\x08location\x18\x01 \x01(\tR\x08locationB\x16\n\x12\x63om.tecton.dataobsP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.dataobs.validation_task_params_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\022com.tecton.dataobsP\001'
  _VALIDATIONTASKPARAMS._serialized_start=299
  _VALIDATIONTASKPARAMS._serialized_end=893
  _DYNAMODATASOURCE._serialized_start=896
  _DYNAMODATASOURCE._serialized_end=1221
  _S3DATASOURCE._serialized_start=1223
  _S3DATASOURCE._serialized_end=1265
# @@protoc_insertion_point(module_scope)
