# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/consumption/consumption.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from tecton_proto.common import id_pb2 as tecton__proto_dot_common_dot_id__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n*tecton_proto/consumption/consumption.proto\x12\x18tecton_proto.consumption\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1ctecton_proto/common/id.proto\"\xb9\x03\n\x0f\x43onsumptionInfo\x12\x46\n\x11time_bucket_start\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x0ftimeBucketStart\x12%\n\x0eunits_consumed\x18\x02 \x01(\x03R\runitsConsumed\x12\x16\n\x06metric\x18\x03 \x01(\tR\x06metric\x12P\n\x07\x64\x65tails\x18\x04 \x03(\x0b\x32\x36.tecton_proto.consumption.ConsumptionInfo.DetailsEntryR\x07\x64\x65tails\x12\x1b\n\tsource_id\x18\x05 \x01(\tR\x08sourceId\x12?\n\x0f\x66\x65\x61ture_view_id\x18\x06 \x01(\x0b\x32\x17.tecton_proto.common.IdR\rfeatureViewId\x12\x33\n\x16online_read_aws_region\x18\x07 \x01(\tR\x13onlineReadAwsRegion\x1a:\n\x0c\x44\x65tailsEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n\x05value\x18\x02 \x01(\tR\x05value:\x02\x38\x01\"\xff\x01\n\x17\x45nrichedConsumptionInfo\x12T\n\x10\x63onsumption_info\x18\x01 \x01(\x0b\x32).tecton_proto.consumption.ConsumptionInfoR\x0f\x63onsumptionInfo\x12\x34\n\x16\x66\x65\x61ture_view_workspace\x18\x03 \x01(\tR\x14\x66\x65\x61tureViewWorkspace\x12*\n\x11\x66\x65\x61ture_view_name\x18\x04 \x01(\tR\x0f\x66\x65\x61tureViewName\x12&\n\x0f\x66\x65\x61ture_view_id\x18\x05 \x01(\tR\rfeatureViewIdJ\x04\x08\x02\x10\x03\x42\x1a\n\x16\x63om.tecton.consumptionP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.consumption.consumption_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\026com.tecton.consumptionP\001'
  _CONSUMPTIONINFO_DETAILSENTRY._options = None
  _CONSUMPTIONINFO_DETAILSENTRY._serialized_options = b'8\001'
  _CONSUMPTIONINFO._serialized_start=136
  _CONSUMPTIONINFO._serialized_end=577
  _CONSUMPTIONINFO_DETAILSENTRY._serialized_start=519
  _CONSUMPTIONINFO_DETAILSENTRY._serialized_end=577
  _ENRICHEDCONSUMPTIONINFO._serialized_start=580
  _ENRICHEDCONSUMPTIONINFO._serialized_end=835
# @@protoc_insertion_point(module_scope)
