# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/args/feature_service.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tecton_proto.common import framework_version_pb2 as tecton__proto_dot_common_dot_framework__version__pb2
from tecton_proto.common import id_pb2 as tecton__proto_dot_common_dot_id__pb2
from tecton_proto.args import basic_info_pb2 as tecton__proto_dot_args_dot_basic__info__pb2
from tecton_proto.args import diff_options_pb2 as tecton__proto_dot_args_dot_diff__options__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\'tecton_proto/args/feature_service.proto\x12\x11tecton_proto.args\x1a+tecton_proto/common/framework_version.proto\x1a\x1ctecton_proto/common/id.proto\x1a\"tecton_proto/args/basic_info.proto\x1a$tecton_proto/args/diff_options.proto\"d\n\x11LoggingConfigArgs\x12\x1f\n\x0bsample_rate\x18\x01 \x01(\x02R\nsampleRate\x12.\n\x13log_effective_times\x18\x02 \x01(\x08R\x11logEffectiveTimes\"V\n\nColumnPair\x12!\n\x0cspine_column\x18\x01 \x01(\tR\x0bspineColumn\x12%\n\x0e\x66\x65\x61ture_column\x18\x02 \x01(\tR\rfeatureColumn\"\xec\x01\n\x1c\x46\x65\x61tureServiceFeaturePackage\x12\x45\n\x12\x66\x65\x61ture_package_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x10\x66\x65\x61turePackageId\x12K\n\x12override_join_keys\x18\x02 \x03(\x0b\x32\x1d.tecton_proto.args.ColumnPairR\x10overrideJoinKeys\x12\x1c\n\tnamespace\x18\x03 \x01(\tR\tnamespace\x12\x1a\n\x08\x66\x65\x61tures\x18\x04 \x03(\tR\x08\x66\x65\x61tures\"\xe5\x03\n\x12\x46\x65\x61tureServiceArgs\x12\x45\n\x12\x66\x65\x61ture_service_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x10\x66\x65\x61tureServiceId\x12\x37\n\x04info\x18\x02 \x01(\x0b\x32\x1c.tecton_proto.args.BasicInfoB\x05\x92M\x02\x10\x01R\x04info\x12\x46\n\x07version\x18\x06 \x01(\x0e\x32%.tecton_proto.common.FrameworkVersionB\x05\x92M\x02\x08\x05R\x07version\x12.\n\x0fprevent_destroy\x18\x07 \x01(\x08\x42\x05\x92M\x02\x08\x01R\x0epreventDestroy\x12Z\n\x10\x66\x65\x61ture_packages\x18\x03 \x03(\x0b\x32/.tecton_proto.args.FeatureServiceFeaturePackageR\x0f\x66\x65\x61turePackages\x12;\n\x16online_serving_enabled\x18\x04 \x01(\x08\x42\x05\x92M\x02\x08\x01R\x14onlineServingEnabled\x12>\n\x07logging\x18\x05 \x01(\x0b\x32$.tecton_proto.args.LoggingConfigArgsR\x07loggingB\x13\n\x0f\x63om.tecton.argsP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.args.feature_service_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\017com.tecton.argsP\001'
  _FEATURESERVICEARGS.fields_by_name['info']._options = None
  _FEATURESERVICEARGS.fields_by_name['info']._serialized_options = b'\222M\002\020\001'
  _FEATURESERVICEARGS.fields_by_name['version']._options = None
  _FEATURESERVICEARGS.fields_by_name['version']._serialized_options = b'\222M\002\010\005'
  _FEATURESERVICEARGS.fields_by_name['prevent_destroy']._options = None
  _FEATURESERVICEARGS.fields_by_name['prevent_destroy']._serialized_options = b'\222M\002\010\001'
  _FEATURESERVICEARGS.fields_by_name['online_serving_enabled']._options = None
  _FEATURESERVICEARGS.fields_by_name['online_serving_enabled']._serialized_options = b'\222M\002\010\001'
  _LOGGINGCONFIGARGS._serialized_start=211
  _LOGGINGCONFIGARGS._serialized_end=311
  _COLUMNPAIR._serialized_start=313
  _COLUMNPAIR._serialized_end=399
  _FEATURESERVICEFEATUREPACKAGE._serialized_start=402
  _FEATURESERVICEFEATUREPACKAGE._serialized_end=638
  _FEATURESERVICEARGS._serialized_start=641
  _FEATURESERVICEARGS._serialized_end=1126
# @@protoc_insertion_point(module_scope)
