# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/materialization/spark_cluster.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tecton_proto.spark_api import jobs_pb2 as tecton__proto_dot_spark__api_dot_jobs__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n0tecton_proto/materialization/spark_cluster.proto\x12\x1ctecton_proto.materialization\x1a!tecton_proto/spark_api/jobs.proto\"\xad\x02\n\x17SparkClusterEnvironment\x12I\n!spark_cluster_environment_version\x18\x01 \x01(\x05R\x1esparkClusterEnvironmentVersion\x12\x65\n\x15job_request_templates\x18\n \x01(\x0b\x32\x31.tecton_proto.materialization.JobRequestTemplatesR\x13jobRequestTemplates\x12T\n\'merged_user_deployment_settings_version\x18\x0c \x01(\x05R#mergedUserDeploymentSettingsVersionJ\x04\x08\x02\x10\nJ\x04\x08\x0b\x10\x0c\"\xc1\x01\n\x13JobRequestTemplates\x12X\n\x13\x64\x61tabricks_template\x18\x02 \x01(\x0b\x32\'.tecton_proto.spark_api.StartJobRequestR\x12\x64\x61tabricksTemplate\x12J\n\x0c\x65mr_template\x18\x03 \x01(\x0b\x32\'.tecton_proto.spark_api.StartJobRequestR\x0b\x65mrTemplateJ\x04\x08\x01\x10\x02*b\n\x08TaskType\x12\x0b\n\x07UNKNOWN\x10\x00\x12\t\n\x05\x42\x41TCH\x10\x01\x12\r\n\tSTREAMING\x10\x02\x12\n\n\x06INGEST\x10\x03\x12\x0c\n\x08\x44\x45LETION\x10\x04\x12\x15\n\x11\x44\x45LTA_MAINTENANCE\x10\x05*x\n\x14\x45xecutionEnvironment\x12\x13\n\x0f\x45NV_UNSPECIFIED\x10\x00\x12\x1b\n\x17\x45NV_DATABRICKS_NOTEBOOK\x10\x01\x12\x0b\n\x07\x45NV_EMR\x10\x02\x12\x11\n\rENV_SNOWFLAKE\x10\x03\x12\x0e\n\nENV_ATHENA\x10\x04\x42\x1e\n\x1a\x63om.tecton.materializationP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.materialization.spark_cluster_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\032com.tecton.materializationP\001'
  _TASKTYPE._serialized_start=617
  _TASKTYPE._serialized_end=715
  _EXECUTIONENVIRONMENT._serialized_start=717
  _EXECUTIONENVIRONMENT._serialized_end=837
  _SPARKCLUSTERENVIRONMENT._serialized_start=118
  _SPARKCLUSTERENVIRONMENT._serialized_end=419
  _JOBREQUESTTEMPLATES._serialized_start=422
  _JOBREQUESTTEMPLATES._serialized_end=615
# @@protoc_insertion_point(module_scope)
