# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/auth/acl.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tecton_proto.common import id_pb2 as tecton__proto_dot_common_dot_id__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1btecton_proto/auth/acl.proto\x12\x11tecton_proto.auth\x1a\x1ctecton_proto/common/id.proto\"^\n\x03\x41\x63l\x12\x1e\n\npermission\x18\x01 \x01(\tR\npermission\x12\x37\n\x0b\x61pi_key_ids\x18\x02 \x03(\x0b\x32\x17.tecton_proto.common.IdR\tapiKeyIdsB;\n\x0f\x63om.tecton.authP\x01Z&github.com/tecton-ai/tecton_proto/auth')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.auth.acl_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\017com.tecton.authP\001Z&github.com/tecton-ai/tecton_proto/auth'
  _ACL._serialized_start=80
  _ACL._serialized_end=174
# @@protoc_insertion_point(module_scope)
