# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/data/onboarding.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\"tecton_proto/data/onboarding.proto\x12\x11tecton_proto.data\"l\n\x10OnboardingStatus\x12\x17\n\x07user_id\x18\x01 \x01(\tR\x06userId\x12?\n\x06status\x18\x02 \x01(\x0e\x32\'.tecton_proto.data.OnboardingStatusEnumR\x06status\"\xdd\x01\n\x1b\x44\x61taPlatformSetupTaskStatus\x12*\n\x11task_display_name\x18\x01 \x01(\tR\x0ftaskDisplayName\x12S\n\x0btask_status\x18\x02 \x01(\x0e\x32\x32.tecton_proto.data.DataPlatformSetupTaskStatusEnumR\ntaskStatus\x12#\n\rerror_message\x18\x03 \x01(\tR\x0c\x65rrorMessage\x12\x18\n\x07\x64\x65tails\x18\x04 \x01(\tR\x07\x64\x65tails*x\n\x14OnboardingStatusEnum\x12\x1d\n\x19ONBOARDING_STATUS_UNKNOWN\x10\x00\x12 \n\x1cONBOARDING_STATUS_INCOMPLETE\x10\x01\x12\x1f\n\x1bONBOARDING_STATUS_COMPLETED\x10\x02*\xa3\x01\n\x1f\x44\x61taPlatformSetupTaskStatusEnum\x12\x17\n\x13TASK_STATUS_UNKNOWN\x10\x00\x12\x1b\n\x17TASK_STATUS_NOT_STARTED\x10\x01\x12\x17\n\x13TASK_STATUS_RUNNING\x10\x02\x12\x19\n\x15TASK_STATUS_SUCCEEDED\x10\x03\x12\x16\n\x12TASK_STATUS_FAILED\x10\x04\x42\x13\n\x0f\x63om.tecton.dataP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.data.onboarding_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\017com.tecton.dataP\001'
  _ONBOARDINGSTATUSENUM._serialized_start=391
  _ONBOARDINGSTATUSENUM._serialized_end=511
  _DATAPLATFORMSETUPTASKSTATUSENUM._serialized_start=514
  _DATAPLATFORMSETUPTASKSTATUSENUM._serialized_end=677
  _ONBOARDINGSTATUS._serialized_start=57
  _ONBOARDINGSTATUS._serialized_end=165
  _DATAPLATFORMSETUPTASKSTATUS._serialized_start=168
  _DATAPLATFORMSETUPTASKSTATUS._serialized_end=389
# @@protoc_insertion_point(module_scope)
