# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/materialization/materialization_task.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tecton_proto.spark_common import libraries_pb2 as tecton__proto_dot_spark__common_dot_libraries__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from tecton_proto.data import feature_view_pb2 as tecton__proto_dot_data_dot_feature__view__pb2
from tecton_proto.args import feature_view_pb2 as tecton__proto_dot_args_dot_feature__view__pb2
from tecton_proto.args import diff_options_pb2 as tecton__proto_dot_args_dot_diff__options__pb2
from tecton_proto.spark_api import jobs_pb2 as tecton__proto_dot_spark__api_dot_jobs__pb2
from tecton_proto.common import fco_locator_pb2 as tecton__proto_dot_common_dot_fco__locator__pb2
from tecton_proto.common import id_pb2 as tecton__proto_dot_common_dot_id__pb2
from tecton_proto.materialization import materialization_states_pb2 as tecton__proto_dot_materialization_dot_materialization__states__pb2
from tecton_proto.materialization import spark_cluster_pb2 as tecton__proto_dot_materialization_dot_spark__cluster__pb2
from tecton_proto.snowflake import location_pb2 as tecton__proto_dot_snowflake_dot_location__pb2
from tecton_proto.online_store_writer import config_pb2 as tecton__proto_dot_online__store__writer_dot_config__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n7tecton_proto/materialization/materialization_task.proto\x12\x1ctecton_proto.materialization\x1a)tecton_proto/spark_common/libraries.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a$tecton_proto/data/feature_view.proto\x1a$tecton_proto/args/feature_view.proto\x1a$tecton_proto/args/diff_options.proto\x1a!tecton_proto/spark_api/jobs.proto\x1a%tecton_proto/common/fco_locator.proto\x1a\x1ctecton_proto/common/id.proto\x1a\x39tecton_proto/materialization/materialization_states.proto\x1a\x30tecton_proto/materialization/spark_cluster.proto\x1a%tecton_proto/snowflake/location.proto\x1a-tecton_proto/online_store_writer/config.proto\"\xb5\x01\n\"MaterializationTaskStateTransition\x12U\n\ntask_state\x18\x01 \x01(\x0e\x32\x36.tecton_proto.materialization.MaterializationTaskStateR\ttaskState\x12\x38\n\ttimestamp\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\ttimestamp\"\xdb\x04\n\x1e\x42\x61tchMaterializationParameters\x12\x46\n\x11window_start_time\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x0fwindowStartTime\x12\x42\n\x0fwindow_end_time\x18\x04 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\rwindowEndTime\x12H\n\x12\x66\x65\x61ture_start_time\x18\x0b \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x10\x66\x65\x61tureStartTime\x12\x44\n\x10\x66\x65\x61ture_end_time\x18\x0c \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x0e\x66\x65\x61tureEndTime\x12\x1d\n\ntile_count\x18\x05 \x01(\x05R\ttileCount\x12@\n\x1dwrite_to_online_feature_store\x18\x08 \x01(\x08R\x19writeToOnlineFeatureStore\x12\x42\n\x1ewrite_to_offline_feature_store\x18\x06 \x01(\x08R\x1awriteToOfflineFeatureStore\x12\x32\n\x15is_overwrite_backfill\x18\t \x01(\x08R\x13isOverwriteBackfill\x12\x32\n\x15is_bootstrap_backfill\x18\n \x01(\x08R\x13isBootstrapBackfillJ\x04\x08\x01\x10\x02J\x04\x08\x02\x10\x03J\x04\x08\x07\x10\x08\"\xc8\x01\n\x1fIngestMaterializationParameters\x12@\n\x1dwrite_to_online_feature_store\x18\x08 \x01(\x08R\x19writeToOnlineFeatureStore\x12\x42\n\x1ewrite_to_offline_feature_store\x18\x06 \x01(\x08R\x1awriteToOfflineFeatureStore\x12\x1f\n\x0bingest_path\x18\r \x01(\tR\ningestPath\"\xae\x01\n\x12\x44\x65letionParameters\x12\x31\n\x15online_join_keys_path\x18\x01 \x01(\tR\x12onlineJoinKeysPath\x12\x33\n\x16offline_join_keys_path\x18\x02 \x01(\tR\x13offlineJoinKeysPath\x12\x16\n\x06online\x18\x03 \x01(\x08R\x06online\x12\x18\n\x07offline\x18\x04 \x01(\x08R\x07offline\"\x84\x01\n\x1a\x44\x65ltaMaintenanceParameters\x12+\n\x11generate_manifest\x18\x01 \x01(\x08R\x10generateManifest\x12\x39\n\nperiod_end\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tperiodEnd\"-\n\x1fStreamMaterializationParametersJ\x04\x08\x01\x10\x02J\x04\x08\x02\x10\x03\"\xd0\x0c\n\x13MaterializationTask\x12O\n\x17materialization_task_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x15materializationTaskId\x12X\n\x17id_feature_view_locator\x18\x10 \x01(\x0b\x32!.tecton_proto.common.IdFcoLocatorR\x14idFeatureViewLocator\x12\x44\n\x1ematerialization_serial_version\x18\x0e \x01(\x05R\x1cmaterializationSerialVersion\x12i\n\x10\x62\x61tch_parameters\x18\x03 \x01(\x0b\x32<.tecton_proto.materialization.BatchMaterializationParametersH\x00R\x0f\x62\x61tchParameters\x12l\n\x11stream_parameters\x18\x04 \x01(\x0b\x32=.tecton_proto.materialization.StreamMaterializationParametersH\x00R\x10streamParameters\x12l\n\x11ingest_parameters\x18\x11 \x01(\x0b\x32=.tecton_proto.materialization.IngestMaterializationParametersH\x00R\x10ingestParameters\x12\x63\n\x13\x64\x65letion_parameters\x18\x13 \x01(\x0b\x32\x30.tecton_proto.materialization.DeletionParametersH\x00R\x12\x64\x65letionParameters\x12|\n\x1c\x64\x65lta_maintenance_parameters\x18\x18 \x01(\x0b\x32\x38.tecton_proto.materialization.DeltaMaintenanceParametersH\x00R\x1a\x64\x65ltaMaintenanceParameters\x12I\n!spark_cluster_environment_version\x18\r \x01(\x05R\x1esparkClusterEnvironmentVersion\x12m\n\x11state_transitions\x18\x08 \x03(\x0b\x32@.tecton_proto.materialization.MaterializationTaskStateTransitionR\x10stateTransitions\x12Q\n&number_of_attempts_from_old_executions\x18\x12 \x01(\x05R!numberOfAttemptsFromOldExecutions\x12#\n\rerror_message\x18\t \x01(\tR\x0c\x65rrorMessage\x12\x39\n\ncreated_at\x18\x0b \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tcreatedAt\x12\x39\n\nupdated_at\x18\x0c \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tupdatedAt\x12\x41\n\x08\x61ttempts\x18\x14 \x03(\x0b\x32%.tecton_proto.materialization.AttemptR\x08\x61ttempts\x12_\n\x0b\x65nrichments\x18\xe8\x07 \x01(\x0b\x32<.tecton_proto.materialization.MaterializationTaskEnrichmentsR\x0b\x65nrichments\x12O\n\rcanary_params\x18\x15 \x01(\x0b\x32*.tecton_proto.materialization.CanaryParamsR\x0c\x63\x61naryParams\x12-\n\x12manually_triggered\x18\x16 \x01(\x08R\x11manuallyTriggered\x12\'\n\x0fmanaged_retries\x18\x17 \x01(\x08R\x0emanagedRetriesB\x0b\n\ttask_typeJ\x04\x08\x02\x10\x03J\x04\x08\x06\x10\x07J\x04\x08\x07\x10\x08J\x04\x08\n\x10\x0bJ\x04\x08\x0f\x10\x10\"\xee\x03\n\x0c\x43\x61naryParams\x12\x1b\n\tcanary_id\x18\x01 \x01(\tR\x08\x63\x61naryId\x12r\n\x14\x63\x61nary_online_config\x18\x02 \x01(\x0b\x32@.tecton_proto.online_store_writer.OnlineStoreWriterConfigurationR\x12\x63\x61naryOnlineConfig\x12G\n canary_streaming_checkpoint_path\x18\x03 \x01(\tR\x1d\x63\x61naryStreamingCheckpointPath\x12\\\n\x18\x63\x61nary_library_overrides\x18\x04 \x03(\x0b\x32\".tecton_proto.spark_common.LibraryR\x16\x63\x61naryLibraryOverrides\x12~\n canary_spark_cluster_environment\x18\x05 \x01(\x0b\x32\x35.tecton_proto.materialization.SparkClusterEnvironmentR\x1d\x63\x61narySparkClusterEnvironment\x12&\n\x0f\x63\x61nary_run_name\x18\x06 \x01(\tR\rcanaryRunName\"\x99\x01\n\x07\x41ttempt\x12\x38\n\x0bworkflow_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\nworkflowId\x12T\n\'has_legacy_materialization_task_attempt\x18\x02 \x01(\x08R#hasLegacyMaterializationTaskAttempt\"\xcb\x02\n)MaterializationTaskAttemptStateTransition\x12\x62\n\rattempt_state\x18\x01 \x01(\x0e\x32=.tecton_proto.materialization.MaterializationTaskAttemptStateR\x0c\x61ttemptState\x12#\n\rstate_message\x18\x03 \x01(\tR\x0cstateMessage\x12[\n\x12termination_reason\x18\x04 \x01(\x0e\x32,.tecton_proto.spark_api.RunTerminationReasonR\x11terminationReason\x12\x38\n\ttimestamp\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\ttimestamp\"\xc3\x08\n\x1aMaterializationTaskAttempt\x12^\n\x1fmaterialization_task_attempt_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x1cmaterializationTaskAttemptId\x12O\n\x17materialization_task_id\x18\x02 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x15materializationTaskId\x12g\n\x15\x65xecution_environment\x18\x0e \x01(\x0e\x32\x32.tecton_proto.materialization.ExecutionEnvironmentR\x14\x65xecutionEnvironment\x12\x15\n\x06run_id\x18\r \x01(\tR\x05runId\x12 \n\x0crun_page_url\x18\x0c \x01(\tR\nrunPageUrl\x12t\n\x11state_transitions\x18\x04 \x03(\x0b\x32G.tecton_proto.materialization.MaterializationTaskAttemptStateTransitionR\x10stateTransitions\x12%\n\x0e\x61ttempt_number\x18\x0b \x01(\x05R\rattemptNumber\x12\x39\n\ncreated_at\x18\t \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tcreatedAt\x12\x39\n\nupdated_at\x18\n \x01(\x0b\x32\x1a.google.protobuf.TimestampR\tupdatedAt\x12G\n\x0e\x63luster_config\x18\x0f \x01(\x0b\x32 .tecton_proto.args.ClusterConfigR\rclusterConfig\x12\x8f\x01\n\x19online_store_copier_tasks\x18\x11 \x03(\x0b\x32T.tecton_proto.materialization.MaterializationTaskAttempt.OnlineStoreCopierTasksEntryR\x16onlineStoreCopierTasks\x12R\n\x0esnowflake_data\x18\x12 \x01(\x0b\x32+.tecton_proto.materialization.SnowflakeDataR\rsnowflakeData\x1a~\n\x1bOnlineStoreCopierTasksEntry\x12\x10\n\x03key\x18\x01 \x01(\tR\x03key\x12I\n\x05value\x18\x02 \x01(\x0b\x32\x33.tecton_proto.materialization.OnlineStoreCopierTaskR\x05value:\x02\x38\x01J\x04\x08\x03\x10\x04J\x04\x08\x05\x10\tJ\x04\x08\x10\x10\x11\"\xe2\x01\n\x1eMaterializationTaskEnrichments\x12q\n\x19spark_cluster_environment\x18\x03 \x01(\x0b\x32\x35.tecton_proto.materialization.SparkClusterEnvironmentR\x17sparkClusterEnvironment\x12\x41\n\x0c\x66\x65\x61ture_view\x18\x04 \x01(\x0b\x32\x1e.tecton_proto.data.FeatureViewR\x0b\x66\x65\x61tureViewJ\x04\x08\x01\x10\x02J\x04\x08\x02\x10\x03\"\xd5\x01\n$OnlineStoreCopierTaskStateTransition\x12N\n\x05state\x18\x01 \x01(\x0e\x32\x38.tecton_proto.materialization.OnlineStoreCopierTaskStateR\x05state\x12#\n\rstate_message\x18\x03 \x01(\tR\x0cstateMessage\x12\x38\n\ttimestamp\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\ttimestamp\"\xc7\x02\n\x15OnlineStoreCopierTask\x12V\n\x08\x61ttempts\x18\x02 \x03(\x0b\x32:.tecton_proto.materialization.OnlineStoreCopierTaskAttemptR\x08\x61ttempts\x12\x1d\n\ntotal_rows\x18\x04 \x01(\x03R\ttotalRows\x12%\n\x0e\x63ompleted_rows\x18\x03 \x01(\x03R\rcompletedRows\x12\x37\n\x18last_progress_request_id\x18\x05 \x01(\tR\x15lastProgressRequestId\x12Q\n\x17last_progress_send_time\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.TimestampR\x14lastProgressSendTimeJ\x04\x08\x01\x10\x02\"\x8f\x01\n\x1cOnlineStoreCopierTaskAttempt\x12o\n\x11state_transitions\x18\x01 \x03(\x0b\x32\x42.tecton_proto.materialization.OnlineStoreCopierTaskStateTransitionR\x10stateTransitions\"\x80\x01\n\rSnowflakeData\x12J\n\rbase_location\x18\x02 \x01(\x0b\x32%.tecton_proto.snowflake.StageLocationR\x0c\x62\x61seLocation\x12\x1d\n\ntotal_rows\x18\x03 \x01(\x03R\ttotalRowsJ\x04\x08\x01\x10\x02\x42\x1e\n\x1a\x63om.tecton.materializationP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.materialization.materialization_task_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\032com.tecton.materializationP\001'
  _MATERIALIZATIONTASKATTEMPT_ONLINESTORECOPIERTASKSENTRY._options = None
  _MATERIALIZATIONTASKATTEMPT_ONLINESTORECOPIERTASKSENTRY._serialized_options = b'8\001'
  _MATERIALIZATIONTASKSTATETRANSITION._serialized_start=579
  _MATERIALIZATIONTASKSTATETRANSITION._serialized_end=760
  _BATCHMATERIALIZATIONPARAMETERS._serialized_start=763
  _BATCHMATERIALIZATIONPARAMETERS._serialized_end=1366
  _INGESTMATERIALIZATIONPARAMETERS._serialized_start=1369
  _INGESTMATERIALIZATIONPARAMETERS._serialized_end=1569
  _DELETIONPARAMETERS._serialized_start=1572
  _DELETIONPARAMETERS._serialized_end=1746
  _DELTAMAINTENANCEPARAMETERS._serialized_start=1749
  _DELTAMAINTENANCEPARAMETERS._serialized_end=1881
  _STREAMMATERIALIZATIONPARAMETERS._serialized_start=1883
  _STREAMMATERIALIZATIONPARAMETERS._serialized_end=1928
  _MATERIALIZATIONTASK._serialized_start=1931
  _MATERIALIZATIONTASK._serialized_end=3547
  _CANARYPARAMS._serialized_start=3550
  _CANARYPARAMS._serialized_end=4044
  _ATTEMPT._serialized_start=4047
  _ATTEMPT._serialized_end=4200
  _MATERIALIZATIONTASKATTEMPTSTATETRANSITION._serialized_start=4203
  _MATERIALIZATIONTASKATTEMPTSTATETRANSITION._serialized_end=4534
  _MATERIALIZATIONTASKATTEMPT._serialized_start=4537
  _MATERIALIZATIONTASKATTEMPT._serialized_end=5628
  _MATERIALIZATIONTASKATTEMPT_ONLINESTORECOPIERTASKSENTRY._serialized_start=5484
  _MATERIALIZATIONTASKATTEMPT_ONLINESTORECOPIERTASKSENTRY._serialized_end=5610
  _MATERIALIZATIONTASKENRICHMENTS._serialized_start=5631
  _MATERIALIZATIONTASKENRICHMENTS._serialized_end=5857
  _ONLINESTORECOPIERTASKSTATETRANSITION._serialized_start=5860
  _ONLINESTORECOPIERTASKSTATETRANSITION._serialized_end=6073
  _ONLINESTORECOPIERTASK._serialized_start=6076
  _ONLINESTORECOPIERTASK._serialized_end=6403
  _ONLINESTORECOPIERTASKATTEMPT._serialized_start=6406
  _ONLINESTORECOPIERTASKATTEMPT._serialized_end=6549
  _SNOWFLAKEDATA._serialized_start=6552
  _SNOWFLAKEDATA._serialized_end=6680
# @@protoc_insertion_point(module_scope)
