# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/common/spark_schema.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n&tecton_proto/common/spark_schema.proto\x12\x13tecton_proto.common\"K\n\nSparkField\x12\x12\n\x04name\x18\x01 \x01(\tR\x04name\x12)\n\x10structfield_json\x18\x02 \x01(\tR\x0fstructfieldJson\"F\n\x0bSparkSchema\x12\x37\n\x06\x66ields\x18\x01 \x03(\x0b\x32\x1f.tecton_proto.common.SparkFieldR\x06\x66ieldsB\x15\n\x11\x63om.tecton.commonP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.common.spark_schema_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\021com.tecton.commonP\001'
  _SPARKFIELD._serialized_start=63
  _SPARKFIELD._serialized_end=138
  _SPARKSCHEMA._serialized_start=140
  _SPARKSCHEMA._serialized_end=210
# @@protoc_insertion_point(module_scope)
