# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/args/repo_metadata.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tecton_proto.common import id_pb2 as tecton__proto_dot_common_dot_id__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n%tecton_proto/args/repo_metadata.proto\x12\x11tecton_proto.args\x1a\x1ctecton_proto/common/id.proto\"\xa6\x01\n\nSourceInfo\x12.\n\x06\x66\x63o_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x05\x66\x63oId\x12\x14\n\x05scope\x18\x03 \x01(\tR\x05scope\x12#\n\rsource_lineno\x18\x04 \x01(\tR\x0csourceLineno\x12\'\n\x0fsource_filename\x18\x05 \x01(\tR\x0esourceFilenameJ\x04\x08\x02\x10\x03\"W\n\x15\x46\x65\x61tureRepoSourceInfo\x12>\n\x0bsource_info\x18\x01 \x03(\x0b\x32\x1d.tecton_proto.args.SourceInfoR\nsourceInfoB\x13\n\x0f\x63om.tecton.argsP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.args.repo_metadata_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\017com.tecton.argsP\001'
  _SOURCEINFO._serialized_start=91
  _SOURCEINFO._serialized_end=257
  _FEATUREREPOSOURCEINFO._serialized_start=259
  _FEATUREREPOSOURCEINFO._serialized_end=346
# @@protoc_insertion_point(module_scope)
