# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/databricks_api/permissions.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n-tecton_proto/databricks_api/permissions.proto\x12\x1btecton_proto.databricks_api\"b\n\x16GroupPermissionsObject\x12\x1d\n\ngroup_name\x18\x01 \x01(\tR\tgroupName\x12)\n\x10permission_level\x18\x02 \x01(\tR\x0fpermissionLevel\"y\n\x12PermissionsRequest\x12\x63\n\x13\x61\x63\x63\x65ss_control_list\x18\x01 \x03(\x0b\x32\x33.tecton_proto.databricks_api.GroupPermissionsObjectR\x11\x61\x63\x63\x65ssControlList\"\xb8\x01\n\x13PermissionsResponse\x12\x1b\n\tobject_id\x18\x01 \x01(\tR\x08objectId\x12\x1f\n\x0bobject_type\x18\x02 \x01(\tR\nobjectType\x12\x63\n\x13\x61\x63\x63\x65ss_control_list\x18\x03 \x03(\x0b\x32\x33.tecton_proto.databricks_api.GroupPermissionsObjectR\x11\x61\x63\x63\x65ssControlListB\x1d\n\x19\x63om.tecton.databricks_apiP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.databricks_api.permissions_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\031com.tecton.databricks_apiP\001'
  _GROUPPERMISSIONSOBJECT._serialized_start=78
  _GROUPPERMISSIONSOBJECT._serialized_end=176
  _PERMISSIONSREQUEST._serialized_start=178
  _PERMISSIONSREQUEST._serialized_end=299
  _PERMISSIONSRESPONSE._serialized_start=302
  _PERMISSIONSRESPONSE._serialized_end=486
# @@protoc_insertion_point(module_scope)
