# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/args/virtual_data_source.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from tecton_proto.args import basic_info_pb2 as tecton__proto_dot_args_dot_basic__info__pb2
from tecton_proto.common import id_pb2 as tecton__proto_dot_common_dot_id__pb2
from tecton_proto.common import spark_schema_pb2 as tecton__proto_dot_common_dot_spark__schema__pb2
from tecton_proto.common import schema_container_pb2 as tecton__proto_dot_common_dot_schema__container__pb2
from tecton_proto.common import framework_version_pb2 as tecton__proto_dot_common_dot_framework__version__pb2
from tecton_proto.common import data_source_type_pb2 as tecton__proto_dot_common_dot_data__source__type__pb2
from tecton_proto.args import data_source_pb2 as tecton__proto_dot_args_dot_data__source__pb2
from tecton_proto.args import diff_options_pb2 as tecton__proto_dot_args_dot_diff__options__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n+tecton_proto/args/virtual_data_source.proto\x12\x11tecton_proto.args\x1a\"tecton_proto/args/basic_info.proto\x1a\x1ctecton_proto/common/id.proto\x1a&tecton_proto/common/spark_schema.proto\x1a*tecton_proto/common/schema_container.proto\x1a+tecton_proto/common/framework_version.proto\x1a*tecton_proto/common/data_source_type.proto\x1a#tecton_proto/args/data_source.proto\x1a$tecton_proto/args/diff_options.proto\"\xdd\n\n\x15VirtualDataSourceArgs\x12L\n\x16virtual_data_source_id\x18\x01 \x01(\x0b\x32\x17.tecton_proto.common.IdR\x13virtualDataSourceId\x12\x37\n\x04info\x18\x02 \x01(\x0b\x32\x1c.tecton_proto.args.BasicInfoB\x05\x92M\x02\x10\x01R\x04info\x12\x46\n\x07version\x18\n \x01(\x0e\x32%.tecton_proto.common.FrameworkVersionB\x05\x92M\x02\x08\x05R\x07version\x12.\n\x0fprevent_destroy\x18\x10 \x01(\x08\x42\x05\x92M\x02\x08\x01R\x0epreventDestroy\x12T\n\x0ehive_ds_config\x18\x03 \x01(\x0b\x32%.tecton_proto.args.HiveDataSourceArgsB\x05\x92M\x02\x08\x06H\x00R\x0chiveDsConfig\x12T\n\x0e\x66ile_ds_config\x18\x04 \x01(\x0b\x32%.tecton_proto.args.FileDataSourceArgsB\x05\x92M\x02\x08\x06H\x00R\x0c\x66ileDsConfig\x12`\n\x12redshift_ds_config\x18\x07 \x01(\x0b\x32).tecton_proto.args.RedshiftDataSourceArgsB\x05\x92M\x02\x08\x06H\x00R\x10redshiftDsConfig\x12\x63\n\x13snowflake_ds_config\x18\x08 \x01(\x0b\x32*.tecton_proto.args.SnowflakeDataSourceArgsB\x05\x92M\x02\x08\x06H\x00R\x11snowflakeDsConfig\x12^\n\x12spark_batch_config\x18\x0b \x01(\x0b\x32\'.tecton_proto.args.SparkBatchConfigArgsB\x05\x92M\x02\x08\x06H\x00R\x10sparkBatchConfig\x12]\n\x11kinesis_ds_config\x18\x05 \x01(\x0b\x32(.tecton_proto.args.KinesisDataSourceArgsB\x05\x92M\x02\x08\x07H\x01R\x0fkinesisDsConfig\x12W\n\x0fkafka_ds_config\x18\x06 \x01(\x0b\x32&.tecton_proto.args.KafkaDataSourceArgsB\x05\x92M\x02\x08\x07H\x01R\rkafkaDsConfig\x12\x61\n\x13spark_stream_config\x18\x0c \x01(\x0b\x32(.tecton_proto.args.SparkStreamConfigArgsB\x05\x92M\x02\x08\x07H\x01R\x11sparkStreamConfig\x12<\n\x06schema\x18\r \x01(\x0b\x32$.tecton_proto.common.SchemaContainerR\x06schema\x12>\n\x04type\x18\t \x01(\x0e\x32#.tecton_proto.common.DataSourceTypeB\x05\x92M\x02\x08\x03R\x04type\x12W\n\x13\x66orced_batch_schema\x18\x0e \x01(\x0b\x32 .tecton_proto.common.SparkSchemaB\x05\x92M\x02\x08\x01R\x11\x66orcedBatchSchema\x12Y\n\x14\x66orced_stream_schema\x18\x0f \x01(\x0b\x32 .tecton_proto.common.SparkSchemaB\x05\x92M\x02\x08\x01R\x12\x66orcedStreamSchemaB\x11\n\x0f\x62\x61tch_ds_configB\x12\n\x10stream_ds_configB\x13\n\x0f\x63om.tecton.argsP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.args.virtual_data_source_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\017com.tecton.argsP\001'
  _VIRTUALDATASOURCEARGS.fields_by_name['info']._options = None
  _VIRTUALDATASOURCEARGS.fields_by_name['info']._serialized_options = b'\222M\002\020\001'
  _VIRTUALDATASOURCEARGS.fields_by_name['version']._options = None
  _VIRTUALDATASOURCEARGS.fields_by_name['version']._serialized_options = b'\222M\002\010\005'
  _VIRTUALDATASOURCEARGS.fields_by_name['prevent_destroy']._options = None
  _VIRTUALDATASOURCEARGS.fields_by_name['prevent_destroy']._serialized_options = b'\222M\002\010\001'
  _VIRTUALDATASOURCEARGS.fields_by_name['hive_ds_config']._options = None
  _VIRTUALDATASOURCEARGS.fields_by_name['hive_ds_config']._serialized_options = b'\222M\002\010\006'
  _VIRTUALDATASOURCEARGS.fields_by_name['file_ds_config']._options = None
  _VIRTUALDATASOURCEARGS.fields_by_name['file_ds_config']._serialized_options = b'\222M\002\010\006'
  _VIRTUALDATASOURCEARGS.fields_by_name['redshift_ds_config']._options = None
  _VIRTUALDATASOURCEARGS.fields_by_name['redshift_ds_config']._serialized_options = b'\222M\002\010\006'
  _VIRTUALDATASOURCEARGS.fields_by_name['snowflake_ds_config']._options = None
  _VIRTUALDATASOURCEARGS.fields_by_name['snowflake_ds_config']._serialized_options = b'\222M\002\010\006'
  _VIRTUALDATASOURCEARGS.fields_by_name['spark_batch_config']._options = None
  _VIRTUALDATASOURCEARGS.fields_by_name['spark_batch_config']._serialized_options = b'\222M\002\010\006'
  _VIRTUALDATASOURCEARGS.fields_by_name['kinesis_ds_config']._options = None
  _VIRTUALDATASOURCEARGS.fields_by_name['kinesis_ds_config']._serialized_options = b'\222M\002\010\007'
  _VIRTUALDATASOURCEARGS.fields_by_name['kafka_ds_config']._options = None
  _VIRTUALDATASOURCEARGS.fields_by_name['kafka_ds_config']._serialized_options = b'\222M\002\010\007'
  _VIRTUALDATASOURCEARGS.fields_by_name['spark_stream_config']._options = None
  _VIRTUALDATASOURCEARGS.fields_by_name['spark_stream_config']._serialized_options = b'\222M\002\010\007'
  _VIRTUALDATASOURCEARGS.fields_by_name['type']._options = None
  _VIRTUALDATASOURCEARGS.fields_by_name['type']._serialized_options = b'\222M\002\010\003'
  _VIRTUALDATASOURCEARGS.fields_by_name['forced_batch_schema']._options = None
  _VIRTUALDATASOURCEARGS.fields_by_name['forced_batch_schema']._serialized_options = b'\222M\002\010\001'
  _VIRTUALDATASOURCEARGS.fields_by_name['forced_stream_schema']._options = None
  _VIRTUALDATASOURCEARGS.fields_by_name['forced_stream_schema']._serialized_options = b'\222M\002\010\001'
  _VIRTUALDATASOURCEARGS._serialized_start=381
  _VIRTUALDATASOURCEARGS._serialized_end=1754
# @@protoc_insertion_point(module_scope)
