# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tecton_proto/data/feature_store.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n%tecton_proto/data/feature_store.proto\x12\x11tecton_proto.data*\xce\x01\n\x19\x46\x65\x61tureStoreFormatVersion\x12(\n$FEATURE_STORE_FORMAT_VERSION_DEFAULT\x10\x00\x12\x31\n-FEATURE_STORE_FORMAT_VERSION_TIME_NANOSECONDS\x10\x01\x12*\n&FEATURE_STORE_FORMAT_VERSION_TTL_FIELD\x10\x02\x12$\n FEATURE_STORE_FORMAT_VERSION_MAX\x10\x02\x1a\x02\x10\x01\x42\x13\n\x0f\x63om.tecton.dataP\x01')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'tecton_proto.data.feature_store_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\017com.tecton.dataP\001'
  _FEATURESTOREFORMATVERSION._options = None
  _FEATURESTOREFORMATVERSION._serialized_options = b'\020\001'
  _FEATURESTOREFORMATVERSION._serialized_start=61
  _FEATURESTOREFORMATVERSION._serialized_end=267
# @@protoc_insertion_point(module_scope)
